#!/usr/bin/env python
import pdb
#pdb.set_trace()
import glob
import os
import numpy as np
from argparse import ArgumentParser
from collections import defaultdict
from multiprocessing import Pool,cpu_count,active_children
from Extract_qname_from_phased_molecule_cut_phase_blocks_v3_hybrid import *
import time
from subprocess import Popen

parser = ArgumentParser(description="Extract reads for all small chunks:")
parser.add_argument('--phase_cut_folder','-cut_folder',help="folder to save cutting profile")
parser.add_argument('--cut_threshold','-c',type=int,help="cutting threshold", default=100000)
parser.add_argument('--chr_start','-start',type=int,help="chromosome start from", default=1)
parser.add_argument('--chr_end','-end',type=int,help="chromosome end by", default=23)
parser.add_argument('--num_threads','-n',type=int,help="number of threads", default=8)
parser.add_argument('--h5_folder','-h5_folder',help="folder to save h5")
parser.add_argument('--sample_name','-s',help="sample name")
parser.add_argument('--mole_len_dict_prev_file','-file1',help="file1")
parser.add_argument('--mole_len_dict_file','-file2',help="file2")
parser.add_argument('--fastq_folder','-f',help="raw fastqs folder")
parser.add_argument('--out_dir','-o_dir',help="out_dir folder")
parser.add_argument("-mole_len_dict_prev_file", "-dict_prev")
parser.add_argument("-mole_len_dict_file", "-dict")

args = parser.parse_args()
script_path = os.path.dirname(os.path.abspath( __file__ ))
code_path = script_path + "/" 

def check_mole_len(cur_mole_file):
    cat_cmd = "cat " + cur_mole_file + " | wc -l "
    mole_len_curr = Popen(cat_cmd,shell=True).wait()
    return mole_len_curr


def Run_extract_reads_for_smallchunks_all(out_dir,phase_cut_folder,cut_threshold,h5_folder,sample_name,fastq_folder,chr_start,chr_end,num_threads,mole_len_dict_prev,mole_len_dict):
    count = 1
    round_times = int(np.ceil((chr_end - chr_start + 1)/num_threads))
    old_list = defaultdict(list)
    for ii in range(0,num_threads):
        for jj in range(chr_start + round_times*ii,chr_start + round_times*ii + round_times):
            if jj <= chr_end:
                old_list[ii+1].append(jj)

    new_list = defaultdict(list)
    times = 1
    for key,val in old_list.items():
        if times%2 == 1:
            new_list[key] = val
        elif times%2 == 0:
            new_list[key] = val[::-1] 
        times += 1
    
    run_list = defaultdict(list)
    for key,val in new_list.items():
        for ii in range(round_times):
            if ii < len(val):
                run_list[ii+1].append(val[ii])

    for key, val in run_list.items():
        pool = Pool(processes=num_threads)
        mole_num_add = 0
        for chr_num in val:
            phased_h5_file = phase_cut_folder + "chr" + str(chr_num) + ".phased_final_cut_by_" +str(cut_threshold)
            PS_flag_dict_cut_file = phase_cut_folder + "chr" + str(chr_num) + ".phased_final_cut_by_" + str(cut_threshold) + "_phase_blocks.p"
            output_dir = out_dir + "chr" + str(chr_num) + "_files_cutPBHC/" 
            mole_qname_dict_file = h5_folder + sample_name + "_chr" + str(chr_num) +  "_qname.p"
            qname_pos_dict_file = h5_folder + sample_name + "_chr" + str(chr_num) + "_qname_pos.p"
            chr_fastq = fastq_folder + "fastq_by_Chr_" + str(chr_num)
            mole_num_start = 1
            mole_num_end = mole_len_dict[chr_num]
            mole_num_add = mole_len_dict_prev[chr_num]
            print("mole_num_start,mole_num_end,mole_num_add:")
            print(mole_num_start,mole_num_end,mole_num_add)
            if os.path.exists(output_dir):
                print("output folder: " + output_dir + ", existing..., delete old folder first")
                rm_cmd = "rm -rf " + output_dir
                Popen(rm_cmd,shell=True).wait()
                os.makedirs(output_dir)
            else:
                os.makedirs(output_dir)
            pool.apply_async(Extract_start,(output_dir,chr_num,phased_h5_file,PS_flag_dict_cut_file,mole_qname_dict_file,qname_pos_dict_file,chr_fastq,mole_num_start,mole_num_end,mole_num_add,"xin"))
            #Extract_start(output_dir,chr_num,phased_h5_file,PS_flag_dict_cut_file,mole_qname_dict_file,qname_pos_dict_file,chr_fastq,mole_num_start,mole_num_end,mole_num_add,"xin")
            mole_num_add += mole_num_end
        pool.close()
        while len(active_children()) > 1:
            time.sleep(0.5)
        pool.join()
    print("All Done~")




if __name__ == "__main__":
    out_dir = args.out_dir
    if os.path.exists(out_dir):
        print("using existing output folder: " + out_dir)
    else:
        os.makedirs(out_dir)
    phase_cut_folder = args.phase_cut_folder
    cut_threshold = args.cut_threshold
    h5_folder= args.h5_folder
    fastq_folder = args.fastq_folder
    chr_start = args.chr_start
    chr_end = args.chr_end
    num_threads = args.num_threads
    sample_name = args.sample_name
    mole_len_dict_prev_file = args.mole_len_dict_prev_file
    mole_len_dict_prev = pickle.load(open(mole_len_dict_prev_file,"rb"))
    mole_len_dict_file = args.mole_len_dict_file
    mole_len_dict = pickle.load(open(mole_len_dict_file,"rb"))
    Run_extract_reads_for_smallchunks_all(out_dir,phase_cut_folder,cut_threshold,h5_folder,sample_name,fastq_folder,chr_start,chr_end,num_threads,mole_len_dict_prev,mole_len_dict)



#!/usr/bin/env python
import pdb
#pdb.set_trace()
import os
import sys
from argparse import ArgumentParser
from multiprocessing import Pool,cpu_count,active_children,Manager
from subprocess import Popen
import time
from Make_supercontig_based_on_HCbk_from_sam_v5 import *
script_path = os.path.dirname(os.path.abspath( __file__ ))
code_path = script_path + "/" 

parser = ArgumentParser(description="Run all:")
parser.add_argument('--chr_start','-start',type=int,help="chromosome start from", default=1)
parser.add_argument('--chr_end','-end',type=int,help="chromosome end by", default=23)
parser.add_argument('--ref_file','-ref',help="ref file")
parser.add_argument('--ref_dir','-dir',help="ref dir")
parser.add_argument('--cut_threshold','-ct',type=int,help="cut threshold: 100000")
parser.add_argument('--out_dir','-o_dir', help="Directory for outputs")
parser.add_argument('--phase_cut_folder','-pc_dir', help="Directory to store phase info")
args = parser.parse_args()
print(code_path)


def align_cmd(ref_file,microcontig_fasta,microcontig_sam,xin):
    try:
        use_cmd = "minimap2 -a " + ref_file + " " + microcontig_fasta + " > " + microcontig_sam 
    except:
        use_cmd = code_path + "minimap2/" + "minimap2 -a " + ref_file + " " + microcontig_fasta + " > " + microcontig_sam 

    Popen(use_cmd,shell=True).wait() 


def align_microcontigs(chr_start,chr_end,ref_file,out_dir):
    pool = Pool(chr_end - chr_start + 1)
    for chr_num in range(chr_start,chr_end + 1):
        microcontig_fasta = out_dir + "Aquila_cutPBHC_minicontig_chr" + str(chr_num) + ".fasta"
        microcontig_sam = out_dir + "Aquila_cutPBHC_minicontig_chr" + str(chr_num) + ".sam"
        pool.apply_async(align_cmd,(ref_file,microcontig_fasta,microcontig_sam,"xin"))
    pool.close()
    while len(active_children()) > 1:
        time.sleep(0.5)
    pool.join()
    print("alignment all done~")


def Concatenate_microcontigs_all(chr_start,chr_end,cut_threshold,out_dir,phase_cut_folder,ref_dir):
    pool = Pool(chr_end - chr_start + 1)
    for chr_num in range(chr_start,chr_end + 1):
        pool.apply_async(Contig_start,(chr_num,cut_threshold,ref_dir,out_dir,phase_cut_folder,"xin"))
    pool.close()
    while len(active_children()) > 1:
        time.sleep(0.5)
    pool.join()


def Concatenate_wgs(chr_start,chr_end,out_dir):
    for chr_num in range(chr_start,chr_end + 1):
        one_contig = out_dir + "Aquila_Contig_chr" + str(chr_num) + ".fasta"
        cat_cmd = "cat " + one_contig  + " >> " + out_dir + "Aquila_contig.fasta"
        Popen(cat_cmd,shell=True).wait()
    print("all done~")






if __name__ == "__main__":
    chr_start = args.chr_start
    chr_end = args.chr_end
    ref_file = args.ref_file
    ref_dir = args.ref_dir
    out_dir = args.out_dir
    cut_threshold = args.cut_threshold
    phase_cut_folder = args.phase_cut_folder
    align_microcontigs(chr_start,chr_end,ref_file,out_dir)
    Concatenate_microcontigs_all(chr_start,chr_end,cut_threshold,out_dir,phase_cut_folder,ref_dir)
    Concatenate_wgs(chr_start,chr_end,out_dir)



#!/usr/bin/env python
from subprocess import Popen
from argparse import ArgumentParser
import os
import sys
script_path = os.path.dirname(os.path.abspath( __file__ ))
code_path = script_path + "/" 
__author__ = "Xin Zhou@Stanford"
parser = ArgumentParser(description="sort bam by qname:")
parser.add_argument('--bam_file','-b', help="Required parameter, BAM file, called by \"longranger align\"",required=True)
parser.add_argument('--out_dir','-o', help="Directory to store Aquila assembly results, default = ./Assembly_results", default="./Asssembly_results")
parser.add_argument('--num_threads_for_samtools_sort','-t',type=int, help="The number of threads you can define for samtools sort, default = 20",default=20)
args = parser.parse_args()

if __name__ == "__main__":
    if len(sys.argv) == 1:
        Popen("python3 " + "Aquila_sortbam.py -h",shell=True).wait()
    else:
        bam_file = args.bam_file
        out_dir = args.out_dir
        num_threads = int(args.num_threads_for_samtools_sort)
        if os.path.exists(out_dir):
            print("using existing output folder: " + out_dir)
        else:
            os.makedirs(out_dir)

        if os.path.exists(out_dir + "/sorted_bam/"):
            print("using existing output folder: " + out_dir + "/sorted_bam/")
        else:
            os.makedirs(out_dir + "/sorted_bam/")

        bam_sorted_file = out_dir + "/sorted_bam/sorted_bam.bam"
        try:
            sort_bam_cmd = "samtools sort -@ " + str(num_threads) + " -n " + bam_file + " -o " + bam_sorted_file
        except:
            sort_bam_cmd = code_path + "samtools/" + "samtools sort -@ " + str(num_threads) + " -n " + bam_file + " -o " + bam_sorted_file

        Popen(sort_bam_cmd,shell=True).wait()
        idx_bam_cmd = "touch "  + out_dir + "/sorted_bam/finish_bam.txt"
        Popen(idx_bam_cmd,shell=True).wait()



import pdb
#pdb.set_trace()
from collections import defaultdict
import pickle
import numpy as np

from argparse import ArgumentParser
import os
import sys
parser = ArgumentParser(description="Variants Calling:")
parser.add_argument('--chr_start','-start',type=int,help="chromosome start from", default=1)
parser.add_argument('--chr_end','-end',type=int,help="chromosome end by", default=23)
parser.add_argument('--var_size','-v',type=int,help="variant size", default=1)
parser.add_argument('--out_dir','-o_dir', help="Directory to store outputs", default='results_/')

args = parser.parse_args()


def Extract_SV_info(hap_SV_file,output_file,chr_num,v_size):
    f = open(hap_SV_file,"r")
    contig_dict = defaultdict(list)
    curr = 0
    for line in f:
        #print(curr)
        curr += 1
        data = line.rsplit()
        if data[0] == "V":
            #print(data)
            mapq = int(data[5])
            repeat = int(data[4])
            cur_chr_num = data[1]
            if cur_chr_num == "chrX":
                cur_chr_num = "chr23"
            if mapq >= 20 and repeat == 1 and cur_chr_num == "chr" + str(chr_num):
                cur_chr_num = data[1]
                ref_start = int(data[2])
                ref_end = int(data[3])
                ref = data[6]
                alt = data[7]
                contig_num = data[8]
                contig_start = int(data[9])
                contig_end = int(data[10])
                strand_dir = data[-1]
                contig_dict[(cur_chr_num,ref_start)].append([cur_chr_num,ref_start, ref_end, ref, alt, contig_num,contig_start,contig_end,strand_dir])

    contig_dict_filter = defaultdict(list)
    count = 0
    for key, val in contig_dict.items():
        if len(val) > 1:
            #print(">1:")
            #print(val)
            count += 1
        else:
            contig_dict_filter[key] = val[0]


    print(len(contig_dict))
    print(len(contig_dict_filter))

    
    SV_dict = defaultdict(list)
    for key, val in contig_dict_filter.items():
        ref = val[3] 
        alt = val[4]
        len_var = abs(len(ref)-len(alt))
        #if len_var >= 50 or (alt == "-" and len_var == 49) or (ref == "-" and len_var == 49):
        #if len_var >= 1 or (alt == "-" and len_var == 0) or (ref == "-" and len_var == 0):
        #if len_var >= v_size or (alt == "-" and len_var == v_size -1 ) or (ref == "-" and len_var == v_size - 1):
        if (ref == "-" and len(alt) >= v_size) or (alt == "-" and len(ref) >= v_size):
            SV_dict[key] = val
        
    pickle.dump(SV_dict, open(output_file,"wb"))

    print(len(SV_dict))
    return SV_dict


def compare_two_haploid_SV(SV_dict_contig_1_file, SV_dict_contig_2_file,use_chr_num,ref_dict_contig_1_file,ref_dict_contig_2_file,out_dir):
    ins_homo_dict = defaultdict(int)
    ins_hetero_dict = defaultdict(int)
    ins_homo_sv = defaultdict(list)
    ins_hetero_sv = defaultdict(list)
    SV_dict_contig_1 = pickle.load(open(SV_dict_contig_1_file,"rb"))
    SV_dict_contig_2 = pickle.load(open(SV_dict_contig_2_file,"rb"))
    ref_dict_contig_1 = pickle.load(open(ref_dict_contig_1_file,"rb"))
    ref_dict_contig_2 = pickle.load(open(ref_dict_contig_2_file,"rb"))
    count_total_share = 0
    homo_key_dict = defaultdict(int)
    count_same_wrong = 0
    for key, val in SV_dict_contig_1.items():
        flag = 0
        if key in SV_dict_contig_2:
            #print(key)
            homo_key_dict[key] = 1
            val_2 = SV_dict_contig_2[key]
            chr_num = val[0]
            start_1 = val[1]
            end_1 = val[2]
            start_2 = val_2[1]
            end_2 = val_2[2]
            ref_1 = val[3]
            ref_2 = val_2[3]
            alt_1 = val[4]
            alt_2 = val_2[4]
            PS_start_1 = int(val[5].split("_PS")[-1].split("_hp1")[0].split(":")[0])
            PS_end_1 = int(val[5].split("_PS")[-1].split("_hp1")[0].split(":")[1])
            PS_start_2 = int(val_2[5].split("_PS")[-1].split("_hp2")[0].split(":")[0])
            PS_end_2 = int(val_2[5].split("_PS")[-1].split("_hp2")[0].split(":")[1])
            if start_1 == start_2 and end_1 == end_2 and ref_1 == ref_2 and alt_1 == alt_2:
                if ref_1 == "-": # ins
                    if PS_end_1 < PS_start_2 or PS_end_2 < PS_start_1:
                        count_same_wrong += 1
                    else:
                        sv_len_1 = len(alt_1)
                        contig_num_1 = val[-4]
                        contig_start_1 = val[-3]
                        contig_end_1 = val[-2]
                        contig_strand_dir_1 = val[-1]
                        contig_num_2 = val_2[-4]
                        contig_start_2 = val_2[-3]
                        contig_end_2 = val_2[-2]
                        contig_strand_dir_2 = val_2[-1]
                        ins_homo_dict[key] = sv_len_1
                        #ins_homo_sv[key] = [chr_num,start_1,end_1,val[5],val[6],val[7],val_2[5],val_2[6],val_2[7]]
                        ins_homo_sv[key] = [chr_num,start_1,end_1,ref_1,alt_1,contig_num_1,contig_start_1,contig_end_1,contig_strand_dir_1,contig_num_2,contig_start_2,contig_end_2,contig_strand_dir_2]

            else:
                if ref_1 == "-": # ins
                    if PS_end_1 < PS_start_2 or PS_end_2 < PS_start_1:
                        count_same_wrong += 1
                    else:
                        sv_len_1 = len(alt_1)
                        sv_len_2 = len(alt_2)
                        contig_num_1 = val[-4]
                        contig_start_1 = val[-3]
                        contig_end_1 = val[-2]
                        contig_strand_dir_1 = val[-1]
                        contig_num_2 = val_2[-4]
                        contig_start_2 = val_2[-3]
                        contig_end_2 = val_2[-2]
                        contig_strand_dir_2 = val_2[-1]
                        ins_homo_dict[key] = np.min([sv_len_1,sv_len_2])
                        #ins_homo_sv[key] = [chr_num,start_1,end_1,val[5],val[6],val[7],val_2[5],val_2[6],val_2[7]]
                        ins_homo_sv[key] = [chr_num,start_1,end_1,ref_1,alt_1,contig_num_1,contig_start_1,contig_end_1,contig_strand_dir_1,contig_num_2,contig_start_2,contig_end_2,contig_strand_dir_2]

        else:
            chr_num = val[0]
            start_1 = val[1]
            end_1 = val[2]
            ref_1 = val[3]
            alt_1 = val[4]
            sv_len = abs(len(ref_1) - len(alt_1)) + 1
            for _step in range(start_1 - 10, start_1 + 10):
                key_shift = (chr_num,_step)
                if key_shift in SV_dict_contig_2:
                    val_2 = SV_dict_contig_2[key_shift]
                    start_2 = val_2[1]
                    end_2 = val_2[2]
                    ref_2 = val_2[3]
                    alt_2 = val_2[4]
                    if abs(end_1 - end_2) <= 10:
                        flag = 1
                        homo_key_dict[key_shift] = 1
                        if ref_1 == "-": # ins
                            sv_len_1 = len(alt_1)
                            sv_len_2 = len(alt_2)
                            contig_num_1 = val[-4]
                            contig_start_1 = val[-3]
                            contig_end_1 = val[-2]
                            contig_strand_dir_1 = val[-1]
                            contig_num_2 = val_2[-4]
                            contig_start_2 = val_2[-3]
                            contig_end_2 = val_2[-2]
                            contig_strand_dir_2 = val_2[-1]
                            ins_homo_dict[key_shift] = np.min([sv_len_1,sv_len_2])
                            #ins_homo_sv[key_shift] = [chr_num,start_1,end_1,val[5],val[6],val[7],val_2[5],val_2[6],val_2[7]]
                            ins_homo_sv[key_shift] = [chr_num,start_1,end_1,ref_1,alt_1,contig_num_1,contig_start_1,contig_end_1,contig_strand_dir_1,contig_num_2,contig_start_2,contig_end_2,contig_strand_dir_2]

                        break
            if flag == 0:
                if ref_1 == "-": # ins
                    count_cov = 0
                    for _step in range(start_1 - 20, start_1 + 20):
                        if _step in ref_dict_contig_2:
                            count_cov += 1
                    if count_cov == 40:
                        sv_len_1 = len(alt_1)
                        ins_hetero_dict[key] = sv_len_1
                        contig_start_1 = val[-3]
                        contig_end_1 = val[-2]
                        contig_num_1 = val[-4]
                        contig_strand_dir_1 = val[-1]
                        #ins_hetero_sv[key] = [chr_num,start_1,end_1,val[5],val[6],val[7],0,1,2]
                        ins_hetero_sv[key] = [chr_num,start_1,end_1,ref_1,alt_1,contig_num_1,contig_start_1,contig_end_1,contig_strand_dir_1,'0_PS0:0_hp0',0,0,0]



    # processing the uniq in contig_2
    for key_2, val_2 in SV_dict_contig_2.items():
        if homo_key_dict[key_2] == 1:
            count_total_share += 1
        else:
            start_2 = val_2[1]
            end_2 = val_2[2]
            ref_2 = val_2[3]
            alt_2 = val_2[4]
            if ref_2 == "-": # ins
                count_cov = 0
                for _step in range(start_1 - 20, start_1 + 20):
                    if _step in ref_dict_contig_1:
                        count_cov += 1
                if count_cov == 40:
                    sv_len_2 = len(alt_2)
                    ins_hetero_dict[key_2] = sv_len_2
                    contig_num_2 = val_2[-4]
                    contig_start_2 = val_2[-3]
                    contig_end_2 = val_2[-2]
                    contig_strand_dir_2 = val_2[-1]
                    #ins_hetero_sv[key_2] = [val_2[0],start_2,end_2,0,1,2,val_2[5],val_2[6],val_2[7]]
                    ins_hetero_sv[key_2] = [val_2[0],start_2,end_2,ref_2,alt_2]
                    ins_hetero_sv[key_2] = [val_2[0],start_2,end_2,ref_2,alt_2,'0_PS0:0_hp0',0,0,0,contig_num_2,contig_start_2,contig_end_2,contig_strand_dir_2]



    print("done")
    pickle.dump(ins_homo_sv, open(out_dir + "ins_homo_sv_chr" + str(use_chr_num) + ".p","wb"))
    pickle.dump(ins_hetero_sv, open(out_dir + "ins_hetero_sv_chr" + str(use_chr_num) + ".p","wb"))
    return count_same_wrong


def Write_vcf_for_SV(fw,ins_homo_file,ins_hetero_file,count_id):
    ins_homo = pickle.load(open(ins_homo_file,"rb"))
    ins_hetero = pickle.load(open(ins_hetero_file,"rb"))

    for key, val in ins_homo.items():
        #print(key)
        chr_num = str(val[0])
        start_ = val[1]
        end_ = val[2]
        ref = val[3]
        alt = val[4]
        GT = "1/1"
        contig_num_1 = val[5]
        contig_start_1 = val[6]
        contig_end_1 = val[7]
        contig_strand_dir_1 = val[8]
        contig_num_2 = val[9]
        contig_start_2 = val[10]
        contig_end_2 = val[11]
        contig_strand_dir_2 = val[12]
        new_contig_num_1 = contig_num_1.replace(":","_")
        new_contig_num_2 = contig_num_2.replace(":","_")
        contig_info = str(new_contig_num_1) + "_" + str(contig_start_1) + "_" + str(contig_end_1) + "_" + contig_strand_dir_1 + "_" +  str(new_contig_num_2) + "_" + str(contig_start_2) + "_" + str(contig_end_2) + "_" + contig_strand_dir_2 
        fw.writelines(chr_num + "\t" + str(start_) + "\t" + "event" + str(count_id) + "\t" + ref + "\t" + alt + "\t" + "." + "\t" + "PASS" + "\t" + "SVTYPE=INS" + "\t" + "GT:Contig" + "\t"  + "1/1"  + ":" + contig_info +  "\n")
        count_id += 1


    for key, val in ins_hetero.items():
        #print(key)
        chr_num = str(val[0])
        start_ = val[1]
        end_ = val[2]
        ref = val[3]
        alt = val[4]
        GT = "0/1"
        contig_num_1 = val[5]
        contig_start_1 = val[6]
        contig_end_1 = val[7]
        contig_strand_dir_1 = val[8]
        contig_num_2 = val[9]
        contig_start_2 = val[10]
        contig_end_2 = val[11]
        contig_strand_dir_2 = val[12]
        new_contig_num_1 = contig_num_1.replace(":","_")
        new_contig_num_2 = contig_num_2.replace(":","_")
        contig_info = str(new_contig_num_1) + "_" + str(contig_start_1) + "_" + str(contig_end_1) + "_" + str(contig_strand_dir_1) + "_" +  str(new_contig_num_2) + "_" + str(contig_start_2) + "_" + str(contig_end_2) + "_" + str(contig_strand_dir_2) 
        fw.writelines(chr_num + "\t" + str(start_) + "\t" + "event" + str(count_id) + "\t" + ref + "\t" + alt + "\t" + "." + "\t" + "PASS" + "\t" + "SVTYPE=INS" + "\t" + "GT:Contig" + "\t"  + "0/1"  + ":"   + contig_info +  "\n")
        count_id += 1

 
    return (fw,count_id)



if __name__ == "__main__":
    out_dir = args.out_dir
    if os.path.exists(out_dir):
        print("using existing output folder: " + out_dir)
    else:
        os.makedirs(out_dir)
    chr_start = args.chr_start
    chr_end = args.chr_end
    v_size = args.var_size

    total_same_wrong = 0
    count_id = 1
    for chr_num in range(chr_start,chr_end + 1):
        final_SV_vcf = out_dir + "Aquila_INS_chr" + str(chr_num)  + ".vcf"
        fw = open(final_SV_vcf,"w")


        SV_dict_contig_1 = Extract_SV_info(out_dir + "Aquila_Contig_chr" + str(chr_num) + "_hp1.var.txt",out_dir + "ins_dict_contig_1_chr" + str(chr_num) + ".p", chr_num,v_size)
        SV_dict_contig_2 = Extract_SV_info(out_dir + "Aquila_Contig_chr" + str(chr_num) + "_hp2.var.txt",out_dir + "ins_dict_contig_2_chr" + str(chr_num) + ".p", chr_num,v_size)
        count_same_wrong = compare_two_haploid_SV(out_dir + "ins_dict_contig_1_chr" + str(chr_num) + ".p", out_dir + "ins_dict_contig_2_chr" + str(chr_num) + ".p", chr_num, out_dir + "ref_dict_contig_1_chr" + str(chr_num) + ".p", out_dir + "ref_dict_contig_2_chr" + str(chr_num) + ".p",out_dir )
        total_same_wrong += count_same_wrong


        fw,count_id = Write_vcf_for_SV(fw,out_dir + "ins_homo_sv_chr" + str(chr_num) + ".p", out_dir + "ins_hetero_sv_chr" +str(chr_num) + ".p",count_id)

        fw.close()
    print("total_same_wrong:")
    print(total_same_wrong)



#!/usr/bin/env python
import pdb
#pdb.set_trace()
import glob
import os
from argparse import ArgumentParser
from multiprocessing import Pool,cpu_count,active_children,Manager
from subprocess import Popen
import time
from Concatenate_contigs_all_v4_extend_for_HCbk import *
import multiprocessing
script_path = os.path.dirname(os.path.abspath( __file__ ))
code_path = script_path + "/" 
parser = ArgumentParser(description="run local assembly by spades:")
parser.add_argument('--num_threads','-nt', help="number of threads",default=30)
parser.add_argument('--num_threads_spades','-nt_spades', help="number of threads for spades",default=5)
parser.add_argument('--out_dir','-o', help="out dir")
parser.add_argument('--minicontig_dir','-minicontig_o', help="minicontig dir")
parser.add_argument('--chr_start','-start',type=int,help="chromosome start from", default=1)
parser.add_argument('--chr_end','-end',type=int,help="chromosome end by", default=23)
args = parser.parse_args()
num_threads_spades = int(args.num_threads_spades)

def use_spades(one_file_fastq,out_dir,xin):
    try:
        use_cmd = code_path + "SPAdes-3.13.0-Linux/bin/" + "spades.py -t " + str(num_threads_spades) + " --only-assembler --12 " + one_file_fastq + " -o " + out_dir 
    except:
        use_cmd = "spades.py -t " + str(num_threads_spades) + " --only-assembler --12 " + one_file_fastq + " -o " + out_dir 

    Popen(use_cmd,shell=True).wait()


def del_one_dir_1(out_dir,xin):
    temp_dir = out_dir + "_temp/"
    use_cmd_1 = "mv " + out_dir + " " + temp_dir
    Popen(use_cmd_1,shell=True).wait()
    use_cmd_2 = "mkdir " + out_dir
    Popen(use_cmd_2,shell=True).wait()
    contig_file = temp_dir + "contigs.fasta"
    if os.path.exists(contig_file):
        use_cmd_3 = "mv " + contig_file + " " + out_dir
        Popen(use_cmd_3,shell=True).wait()
    rm_cmd = "rm -rf " + temp_dir
    Popen(rm_cmd,shell=True).wait()


def del_one_dir_2(out_dir,xin):
    rm_cmd = "rm -rf " + out_dir
    Popen(rm_cmd,shell=True).wait()


def run_spades_all(chr_start,chr_end,output_dir,num_of_threads,minicontig_dir):
    num_of_threads = multiprocessing.cpu_count() - 10
    for chr_num in range(chr_start, chr_end+1):
        in_dir = output_dir + "chr" + str(chr_num) + "_files_cutPBHC/" 
        count = 1
        fastq_files_all = sorted(glob.glob(in_dir +  "fastq_by_*.fastq"),key=os.path.getsize,reverse=True)
        total_num = len(fastq_files_all)
        pool = Pool(num_of_threads)
        out_dir_list = []
        for one_file_fastq in fastq_files_all:
            one_file = one_file_fastq[:-6]
            out_dir = one_file + "_spades_assembly"
            spades_contig_file = out_dir + "/" + "contigs.fasta"
            if os.path.exists(spades_contig_file):
                count += 1
                #print("using existing " + spades_contig_file)
                out_dir_list.append(out_dir)
            else:
                count += 1
                pool.apply_async(use_spades,(one_file_fastq,out_dir,"xin"))
                out_dir_list.append(out_dir)
            if (count - 1)%num_of_threads == 0 or (count - 1) == total_num:
                pool.close()
                while len(active_children()) > 1:
                    time.sleep(0.5)
                pool.join()
                
                # delete temp files
                """
                if out_dir_list != []:
                    pool = Pool(num_of_threads)
                    for one_dir in out_dir_list:
                        pool.apply_async(del_one_dir_1,(one_dir,"xin"))
                    pool.close()
                    while len(active_children()) > 1:
                        time.sleep(0.5)
                    pool.join()
                """

                if (count - 1) == total_num:
                    print("finished chr" + str(chr_num))
                else:
                    pool = Pool(num_of_threads)
                    out_dir_list = []
        output_file = "Aquila_cutPBHC_minicontig" + "_chr" + str(chr_num) + ".fasta"
        Concatenate_start(in_dir,minicontig_dir,output_file,"xin")   
        
        # delete assembly files
        """
        time.sleep(5)
        pool = Pool(num_of_threads)
        count = 1
        for one_file_fastq in fastq_files_all:
            one_file = one_file_fastq[:-6]
            out_dir = one_file + "_spades_assembly"
            count += 1
            pool.apply_async(del_one_dir_2,(out_dir,"xin"))
            if (count - 1)%num_of_threads == 0 or (count - 1) == total_num:
                pool.close()
                while len(active_children()) > 1:
                    time.sleep(0.5)
                pool.join()
                if (count - 1) == total_num:
                    print("finished chr" + str(chr_num))
                else:
                    pool = Pool(num_of_threads)
        time.sleep(5)
        """
    print("All Done~")




if __name__ == "__main__":
    output_dir = args.out_dir
    minicontig_dir = args.minicontig_dir
    chr_start = args.chr_start
    chr_end = args.chr_end
    num_of_threads = int(args.num_threads)
    run_spades_all(chr_start,chr_end,output_dir,num_of_threads,minicontig_dir)




#!/usr/bin/env python
import pdb
#pdb.set_trace()
import os
import numpy as np
from argparse import ArgumentParser
from Concatenate_contigs_all_v4_extend_for_HCbk import *
from multiprocessing import Pool,cpu_count,active_children,Manager
import time
script_path = os.path.dirname(os.path.abspath( __file__ ))

code_path = script_path + "/" 

parser = ArgumentParser(description="Run microcontigs all:")
parser.add_argument('--chr_start','-start',type=int,help="chromosome start from", default=1)
parser.add_argument('--chr_end','-end',type=int,help="chromosome end by", default=23)
parser.add_argument('--in_dir','-i_dir', help="Directory for inputs")
parser.add_argument('--out_dir','-o_dir', help="Directory to store outputs", default='results_/')
parser.add_argument('--output_prefix','-output_prefix', help="output file")

args = parser.parse_args()


def Run_all(chr_start,chr_end,in_dir,out_dir,output_file):
    pool = Pool(chr_end - chr_start + 1)
    for chr_num in range(chr_start,chr_end+1):
        output_file = output_prefix + "_chr" + str(chr_num) + ".fasta"
        in_dir_2 = in_dir + "chr" + str(chr_num) + "_files_cutPBHC/"
        pool.apply_async(Concatenate_start,(in_dir_2,out_dir,output_file,"xin"))
    pool.close()
    while len(active_children()) > 1:
        time.sleep(0.5)
    pool.join()

    print("all done~")

   


if __name__ == "__main__":
    out_dir = args.out_dir
    if os.path.exists(out_dir):
        print("using existing output folder: " + out_dir)
    else:
        os.makedirs(out_dir)
    chr_start = args.chr_start
    chr_end = args.chr_end
    in_dir = args.in_dir
    out_dir = args.out_dir
    output_prefix= args.output_prefix
    Run_all(chr_start,chr_end,in_dir,out_dir,output_prefix)
 



import pdb
#pdb.set_trace()
import glob
import os
import numpy as np
from argparse import ArgumentParser
import pickle


def concatenate_contigs(contig_file,fw,contig_num,contigs_all,hp_flag,PS_flag,PS_flag_2):
    f = open(contig_file,"r")
    contig_ = ""
    flag = 0
    for line in f:
        data = line.rsplit()
        if data[0][0] == ">":
            if flag == 1:
                fw.writelines(">" + str(contig_num) + "_PS" +  str(PS_flag) + ":" + str(PS_flag_2) + "_hp" + str(hp_flag) + "\n")
                fw.writelines(contig_ + "\n")
                contigs_all.append(len(contig_))
                contig_num += 1
                contig_ = ""
                flag = 0
            flag = 1
        else:
            string_ = data[0]
            contig_ += string_

    fw.writelines(">" + str(contig_num) + "_PS" +  str(PS_flag) + ":" + str(PS_flag_2) + "_hp" + str(hp_flag) + "\n")
    fw.writelines(contig_ + "\n")
    contig_num += 1
    return (fw,contig_num,contigs_all)


def concatenate_all(input_dir,output_filename):
    count = 0
    count_2 = 0
    count_3 = 0
    contig_num = 1
    contigs_all = []

    fw_contigs_all = open(output_filename,"w")
    for one_file in glob.glob(input_dir + "fastq_by_*_spades_assembly"):
        #print(one_file)
        count += 1
        spades_contig_file = one_file + "/" + "contigs.fasta"
        if os.path.exists(spades_contig_file):
            if "hp1" in one_file:
                hp_flag = 1
            else:
                hp_flag = 2
            count_2 += 1
            PS_flag = int(one_file[len(input_dir):].split("fastq_by_")[1].split("_")[0])
            PS_flag_2 = int(one_file[len(input_dir):].split("fastq_by_")[1].split("_")[1])
            fw_contigs_all, contig_num,contigs_all = concatenate_contigs(spades_contig_file,fw_contigs_all,contig_num,contigs_all,hp_flag,PS_flag,PS_flag_2)
        else:
            count_3 += 1

    print(count,count_2,count_3)
    print(np.mean(contigs_all))
    print(np.median(contigs_all))
    print(np.max(contigs_all))
    print(np.min(contigs_all))

    contigs_all_sorted = sorted(contigs_all,reverse=True)
    total_contigs_len = sum(contigs_all)
    total_contigs_len_half = float(total_contigs_len)/2
    cumu_len = 0
    for one_len in contigs_all_sorted:
        cumu_len += one_len
        if cumu_len >= total_contigs_len_half:
            n50_len = one_len
            print("------results:-------")
            print("n50 minicontigs: " + str(n50_len))
            break

    fw_contigs_all.close()    


def Concatenate_start(input_dir,output_dir,output_file,xin): 
    if os.path.exists(output_dir):
        print("using existing output folder: " + output_dir)
    else:
        os.makedirs(output_dir)

    output_filename = output_dir + output_file
    concatenate_all(input_dir,output_filename)




import pdb
from subprocess import Popen
from argparse import ArgumentParser
import os
import sys

parser = ArgumentParser(description="Author: xzhou15@cs.stanford.edu\n",usage='use "python3 %(prog)s --help" for more information')
parser.add_argument('--chr_start','-start',type=int,help="chromosome start from", default=1)
parser.add_argument('--chr_end','-end',type=int,help="chromosome end by", default=23)
parser.add_argument('--assembly_dir','-i', help="Directory to store assembly results",required=True)

args = parser.parse_args()

def Cat_Contigs_for_wgs(assembly_dir,chr_start,chr_end):
    in_dir = assembly_dir + "Assembly_Contigs_files/"
    for chr_num in range(chr_start,chr_end + 1):
        one_contig = in_dir + "Aquila_Contig_chr" + str(chr_num) + ".fasta"
        cat_cmd = "cat " + one_contig  + " >> " + in_dir + "Aquila_Contig_WGS.fasta"
        Popen(cat_cmd,shell=True).wait()
    print("all done~")



if __name__ == "__main__":
    if len(sys.argv) == 1:
        Popen("python3 " + "Aquila_Contigs_WGS.py -h",shell=True).wait()
    else:
        chr_start = args.chr_start
        chr_end = args.chr_end
        assembly_dir = args.assembly_dir + "/"
        Cat_Contigs_for_wgs(assembly_dir,chr_start,chr_end)












#!/usr/bin/env python
import pdb
#pdb.set_trace()
from subprocess import Popen
from argparse import ArgumentParser
import os
import sys
import pickle
import os.path
script_path = os.path.dirname(os.path.abspath( __file__ ))
code_path = script_path + "/" 
__author__ = "Xin Zhou@Stanford"
parser = ArgumentParser(description="Author: xzhou15@cs.stanford.edu\n",usage='use "python3 %(prog)s --help" for more information')
parser.add_argument('--chr_start','-start',type=int,help="chromosome start from, default = 1", default=1)
parser.add_argument('--chr_end','-end',type=int,help="chromosome end by, default = 23", default=23)
parser.add_argument('--out_dir','-o', help="Required parameter; Directory to store assembly results",required=True)
parser.add_argument('--reference','-ref', help="Required parameter; reference fasta file, run ./install to download \"source/ref.fa\" for GRCh38",required=True)
parser.add_argument('--num_threads','-t',type=int,help="number of threads, default = 30, this correponds to number of small files get assembled simulateoulsy", default=30)
parser.add_argument('--num_threads_spades','-t_spades',type=int,help="number of threads for spades, default = 5", default=5)
parser.add_argument('--block_len_use','-bl',type=int,help="phase block len threshold, default = 100000",default=100000)

args = parser.parse_args()

def read_ref(fasta_file,chr_num,out_dir):
    f = open(fasta_file,"r")
    count = 0
    ref_seq = ""
    for line in f:
        if count > 0:
            data = line.rsplit()
            ref_seq += data[0]
        count += 1
    print("total_len for chr" + str(chr_num))
    print(len(ref_seq))
    pickle.dump(ref_seq, open(out_dir + "ref_seq_chr" + str(chr_num) +  ".p","wb"))


def extract_ref_chr(ref_file,chr_num,out_dir):
    fw = open(out_dir + "genome_ref_chr" + str(chr_num) + ".fasta","w")
    f = open(ref_file,"r")
    flag = 0
    total_len = 0
    for line in f:
        data = line.rsplit()
        if data[0] == ">chr" + str(chr_num):
            fw.writelines(">" + str(chr_num) + "\n")
            flag = 1
        elif data[0][0] == ">" and flag == 1:
            break
        else:
            if flag == 1:
                total_len += len(data[0])
                fw.writelines(data[0] + "\n")
    print("chr" + str(chr_num) + ":")
    print(total_len)


def local_assembly_for_small_chunks(chr_start,chr_end,num_threads,num_threads_spades,Local_Assembly_dir,Assembly_Contigs_dir):
    use_cmd = "python " + code_path + "Run_spades_final_MT_2_all_noec_deltemp.py" + " --num_threads " + str(num_threads) + " --num_threads_spades " + str(num_threads_spades) + " --chr_start " + str(chr_start) + " --chr_end " + str(chr_end) + " --out_dir " + Local_Assembly_dir + " --minicontig_dir " + Assembly_Contigs_dir
    Popen(use_cmd,shell=True).wait()


def Complete_contiguity(chr_start,chr_end,Assembly_Contigs_dir,phase_blocks_cut_highconf_dir,cut_threshold,ref_dir,ref_idx_file):
    use_cmd = "python " + code_path + "Concatenate_contigs_from_microcontigs.py" + " --chr_start "  + str(chr_start) + " --chr_end " + str(chr_end) + " --out_dir " + Assembly_Contigs_dir + " --phase_cut_folder " + phase_blocks_cut_highconf_dir + " --cut_threshold " + str(cut_threshold) + " --ref_file " + ref_idx_file + " --ref_dir " + ref_dir
    Popen(use_cmd,shell=True).wait()
  


def main():
    if len(sys.argv) == 1:
        Popen("python " + "Aquila_step2.py -h",shell=True).wait()
    else:
        chr_start = args.chr_start
        chr_end = args.chr_end
        ref_file = args.reference
        cut_threshold = args.block_len_use
        num_threads = int(args.num_threads)
        num_threads_spades = int(args.num_threads_spades)
        ref_dir = args.out_dir + "/ref_dir/"
        if os.path.exists(ref_dir):
            print("using existing output folder: " + ref_dir)
        else:
            os.makedirs(ref_dir)
        if ~os.path.exists(ref_dir + "ref.mmi"):
            try:
                mk_ref_idx = "minimap2 -d " + ref_dir + "ref.mmi "  + ref_file
            except:
                mk_ref_idx = code_path + "minimap2/" + "minimap2 -d " + ref_dir + "ref.mmi "  + ref_file

            Popen(mk_ref_idx,shell=True).wait()
        for chr_num in range(chr_start,chr_end + 1):
            extract_ref_chr(ref_file,chr_num,ref_dir)
        extract_ref_chr(ref_file,"X",ref_dir)
        for chr_num in range(chr_start,chr_end + 1):
            read_ref(ref_dir + "genome_ref_chr" + str(chr_num) + ".fasta",chr_num,ref_dir)
        read_ref(ref_dir + "genome_ref_chrX.fasta",23,ref_dir)

        ref_idx_file = ref_dir + "ref.mmi"
        Assembly_Contigs_dir = args.out_dir + "/Assembly_Contigs_files/"
        phase_blocks_cut_highconf_dir = args.out_dir + "/phase_blocks_cut_highconf/"
        Local_Assembly_dir = args.out_dir + "/Local_Assembly_by_chunks/"

        local_assembly_for_small_chunks(chr_start,chr_end,num_threads,num_threads_spades,Local_Assembly_dir,Assembly_Contigs_dir)
        Complete_contiguity(chr_start,chr_end,Assembly_Contigs_dir,phase_blocks_cut_highconf_dir,cut_threshold,ref_dir,ref_idx_file)
    

if __name__ == "__main__":
	main()






import pickle
from collections import defaultdict
import numpy as np


def read_phase_block_file(pickle_file,h5_file,output_file,metric_phase_percent,metric_corr_percent,hetero_var_dict):
    phase_block = pickle.load(open(pickle_file,"rb"))
    phase_block_2 = [] 
    total_num = len(phase_block)

    for _num in range(total_num):
        var_dict_2 = defaultdict(int)
        var_dict = phase_block[_num]
        for pos, hp in var_dict.items():
            if hp == 1 or hp == 2:
                var_dict_2[pos] = hp
                
        phase_block_2.append(var_dict_2)

    uniq_flag,convert_dict = Assign_phase_block(phase_block_2,h5_file,output_file,metric_phase_percent,metric_corr_percent)
    Refine_phase_block_for_output(uniq_flag,convert_dict,phase_block_2,h5_file,hetero_var_dict)


def get_complement_hp(hp):
    if hp == 1:
        return 2
    elif hp == 2:
        return 1


def get_complement_hp_2(hp):
    if hp == 1:
        return 0
    elif hp == 0:
        return 1


def get_complement_hp_3(hp):
    if hp == "hp1":
        return "hp2"
    elif hp == "hp2":
        return "hp1"


def Assign_phase_block(phase_block_new,h5_file,output_file,metric_phase_percent,metric_corr_percent):
    convert_dict = defaultdict(int)
    uniq_flag = defaultdict(int)
    fw = open(output_file,"w")
    total_blocks = len(phase_block_new)
    prob_dict = defaultdict(lambda: defaultdict(lambda: defaultdict(float)))
    set_HP_tag = 1001   # use an artificial HP_tag first
    for _num in range(total_blocks):
        one_block = phase_block_new[_num]
        PS_tag = set_HP_tag
        set_HP_tag += 1
        convert_dict[PS_tag] = _num
        HP_tag = 'hp1'
        HP_complement_tag = 'hp2'
        for var_pos, var_hp in one_block.items():
            prob_dict[var_pos][var_hp][(PS_tag,HP_tag)] = 0.99
            prob_dict[var_pos][get_complement_hp(var_hp)][(PS_tag,HP_complement_tag)] = 0.99

    count_phased = 0
    count_phased_error = 0
    #print(prob_dict)    
    f = open(h5_file, "r")
    curr = 0
    final_correct_percent = []
    for line in f:
        data = line.rsplit()
       # print(curr)
        curr += 1
        data = line.rsplit()
        if data[0] == "X":
            chr_num = 23
        else:
            chr_num = int(data[0])
        mole_num = int(data[6])
        mole_start = int(data[1])
        mole_end = int(data[2])
        var_list = data[7:]
        var_dict = defaultdict(int)
        for var in var_list:
            var_info = var.split(":")
            var_pos = int(var_info[0])
            var_hp = int(var_info[1])
            var_dict[var_pos] = var_hp + 1     # add 1 for operation

        mole_prob = defaultdict(int)
        for var_pos, var_hp in var_dict.items():
            prob_info = prob_dict[var_pos]
            if prob_info != {}:
                PS_info = prob_info[var_hp]
                for PS_tag_all, prob in PS_info.items():
                    mole_prob[PS_tag_all] += 1

        if len(mole_prob) >= 1:
            max_num = 0
            for PS_tag, num_of_prob in mole_prob.items():
                if max_num < num_of_prob:
                    use_PS_tag = PS_tag
                    max_num = num_of_prob

            min_num = mole_prob[(use_PS_tag[0],get_complement_hp_3(use_PS_tag[1]))]
            if max_num >= 3*min_num:   # may need update this threshold
                final_correct_percent.append(float(max_num)/(max_num + min_num))
                #print(curr, len(var_dict))
                #print(mole_prob)
                #print("------------------------------------------")
                #print(mole_prob[use_PS_tag])
                #print(min_num)
                #print(use_PS_tag)
                count_phased += 1
                #print(" ")
                fw.writelines("\t".join(data) + "\t" + "PS:" + str(use_PS_tag[0]) + "\t" + use_PS_tag[1] + "\n")
                if len(mole_prob) > 2:
                    for pos_,hp_ in var_dict.items():
                        _info = prob_dict[pos_][hp_]
                        if len(_info) > 1:
                            uniq_flag[pos_] = use_PS_tag


            else:
                count_phased_error += 1
                fw.writelines(line)

        else:
            fw.writelines(line)

    #print("---result---")
    metric_phase_percent.append(float(count_phased)/curr)
    metric_corr_percent.append(np.mean(final_correct_percent))
    #print(count_phased,count_phased_error,curr,float(count_phased)/curr)
    #print(np.mean(final_correct_percent))
    #print(len(phase_block_new))
    #return (metric_phase_percent,metric_corr_percent)
    return (uniq_flag,convert_dict)


def Refine_phase_block_for_output(uniq_flag,convert_dict,phase_block,h5_file,hetero_var_dict):
    output_file = h5_file + "_final.p"
    count_total = 0
    var_count = defaultdict(int)
    print(len(uniq_flag))
    num_of_blocks = len(phase_block)
    for _num in range(num_of_blocks):
        block_ = phase_block[_num]
        for _pos, _hp in block_.items():
            var_count[_pos] += 1
    for _pos, _count in var_count.items():
        if _count > 1:
            count_total += 1

    print(count_total)
    phase_block_eval = []
    # use the uniq_flag for overlap variants
    # for the final phase blocks, refine some overlap variants when maximize the haplotype likelihood of all molecules 
    chr_num_raw = h5_file.split("chr")[1].split("_")[0]
    if chr_num_raw == "23":
        chr_num = "X"
    else:
        chr_num = int(chr_num_raw)
    for _num in range(num_of_blocks):
        block_new = defaultdict(int)
        block_ = phase_block[_num]
        for _pos, _hp in block_.items():
            if _pos in uniq_flag.keys():
                use_flag = uniq_flag[_pos]
                if convert_dict[use_flag[0]] == _num:
                    block_new[_pos] = _hp
            else:
                block_new[_pos] = _hp

        if block_new != {}:
            phase_block_eval.append(block_new)

    # use the refined phase blocks to do evaluation with gold standard
    # extract gold standard haplotype
    num_of_blocks_new = len(phase_block_eval)
    total_val = 0
    count_nofound = 0
    count_found = 0
    phase_block_eval_new = []
    count_wrong = 0
    for jj in range(num_of_blocks_new):
        block = phase_block_eval[jj]
        new_block = defaultdict(list)
        for _pos,_val in block.items():
            if (chr_num,_pos) in hetero_var_dict:
                new_block[_pos] = [_val,hetero_var_dict[(chr_num,_pos)][0],hetero_var_dict[(chr_num,_pos)][1]]
            else:
                count_wrong += 1
        phase_block_eval_new.append(new_block)  
        total_val += len(block)
    print(total_val)
    pickle.dump(phase_block_eval_new,open(output_file,"wb"))
    

def Finalize_phase_block(output_file_raw, output_file):
    PS_flag_dict = defaultdict(int)
    count_mole = defaultdict(int)
    f = open(output_file_raw, "r")
    fw = open(output_file, "w")
    for line in f:
        data = line.rsplit()
        var_dict = []
        if data[-1] == "hp1" or data[-1] == "hp2":
            PS_tag = data[-2]
            PS_flag = int(PS_tag.split(":")[1])
            HP_tag = data[-1]
            var_list = data[7:-2]
            count_ = 0
            for var in var_list:
                var_pos = int(var.split(":")[0])
                if count_ == 0:
                    min_pos = var_pos
                else:
                    if min_pos > var_pos:
                        min_pos = var_pos
                count_ += 1

            if count_mole[PS_flag] == 0:
                PS_flag_dict[PS_flag] = min_pos
            else:
                if min_pos < PS_flag_dict[PS_flag]:
                    PS_flag_dict[PS_flag] = min_pos

            count_mole[PS_flag] += 1

    f.close()
    f = open(output_file_raw, "r")
    for line in f:
        data = line.rsplit()
        if data[-1] == "hp1" or data[-1] == "hp2":
            PS_tag = data[-2]
            PS_flag = int(PS_tag.split(":")[1])
            HP_tag = data[-1]
            data[-2] = "PS:" + str(PS_flag_dict[PS_flag])
            fw.writelines("\t".join(data) + "\n")
        else:
            fw.writelines(line)

    f.close()
    fw.close()

    return PS_flag_dict


def Calculate_GenotypeProb_for_variants(phase_block_file):
    prob_dict = defaultdict(lambda: defaultdict(lambda: defaultdict(int)))
    f = open(phase_block_file,"r")
    curr = 0
    for line in f:
        #print(curr)
        curr += 1
        data = line.rsplit()
        flag = data[-1]
        if flag == "hp1" or flag == "hp2":
            var_list = data[7:-2]
            PS_tag = int(data[-2].split(":")[1])
            HP_tag = data[-1]
            HP_complement_tag = get_complement_hp_3(HP_tag)
            for one_var in var_list:
                var_info = one_var.split(":")
                var_pos = int(var_info[0])
                var_hp = int(var_info[1]) + 1    # add 1 for operation
                prob_dict[var_pos][var_hp][(PS_tag,HP_tag)] += 1
                prob_dict[var_pos][get_complement_hp(var_hp)][(PS_tag,HP_complement_tag)] += 1

    f.close()
    return prob_dict


def Assign_phase_block_for_mole_with_one_var(prob_dict,h5_one_var,output_file,nonphase_dict):
    fw = open(output_file,"w")
    f = open(h5_one_var,"r")
    for line in f:
        data = line.rsplit()
        var = data[-1]
        var_info = var.split(":")
        var_pos = int(var_info[0])
        var_hp = int(var_info[1]) + 1   # add 1 for operation

        prob_info = prob_dict[var_pos]
        if prob_info != {}:
            PS_info = prob_info[var_hp]
            curr = 0
            for PS_tag_all, count_ in PS_info.items():
                if curr == 0:
                    count_max = count_
                    PS_HP_use = PS_tag_all
                else:
                    if count_ > count_max:
                        PS_HP_use = PS_tag_all

                curr += 1
            PS_tag_use = PS_HP_use[0]
            HP_tag_use = PS_HP_use[1]
            fw.writelines("\t".join(data) + "\t" + "PS:" + str(PS_tag_use) + "\t" + HP_tag_use + "\n")
        else:
            fw.writelines(line)
            nonphase_dict[data[6]] = data[7:]


    fw.close()
    f.close()
    return nonphase_dict


def Impute_phase_block(prob_dict,phase_file,output_file,nonphase_dict):
    f = open(phase_file,"r")
    fw = open(output_file,"w")
    for line in f:
        data = line.rsplit()
        flag = data[-1]
        if flag != "hp1" and flag != "hp2":
            var_list = data[7:]
            mole_prob = defaultdict(int)
            key_use = []
            for one_var in var_list:
                var_pos = int(one_var.split(":")[0])
                var_hp = int(one_var.split(":")[1]) + 1 # add 1 for operation
                prob_info = prob_dict[var_pos]
                if prob_info != {}:
                    PS_info = prob_info[var_hp]
                    curr_ = 0
                    for PS_tag_all, count_ in PS_info.items():
                        if curr_ == 0:
                            count_max = count_
                            PS_HP_use = PS_tag_all
                        else:
                            if count_ > count_max:
                                PS_HP_use = PS_tag_all

                        curr_ += 1

                    PS_tag_use = PS_HP_use[0]
                    HP_tag_use = PS_HP_use[1]

                    mole_prob[(PS_tag_use,HP_tag_use)] += 1

                    count_val = 0 
                    for key, val in mole_prob.items():
                        if count_val == 0:
                            val_max = val
                            key_use = key
                        else:
                            if val > val_max:
                                val_max = val
                                key_use = key
                        count_val += 1
            if key_use == []:
                fw.writelines(line)
                nonphase_dict[data[6]] = data[7:]
            else:
                fw.writelines("\t".join(data) + "\t" + "PS:" + str(key_use[0]) + "\t" + key_use[1] + "\n")
        else:
            fw.writelines(line)

    f.close()
    fw.close()
    return nonphase_dict


def write_phase_block_into_h5(phase_block_final,phase_file,output_file):
    f = open(phase_file,"r")
    fw = open(output_file,"w")
    for line in f:
        data = line.rsplit()
        flag = data[-1]
        if flag != "hp1" and flag != "hp2":
            mole_num = int(data[6])
            phase_info = phase_block_final[mole_num]
            use_PS_tag = phase_info[0]
            use_HP_tag = phase_info[1]
            fw.writelines("\t".join(data) + "\t" + "PS:" + str(use_PS_tag) + "\t" + use_HP_tag + "\n")
        else:
            fw.writelines(line)
    f.close()
    fw.close()
    

def Impute_nonphase_variant(nonphase_dict,phase_file_1,phase_file_2,output_file_1,output_file_2):
    ###nonphase_dict = pickle.load(open("nonphase_dict.p", "rb"))
    merge_dict = defaultdict(list)
    mole_var_pos_min_dict = defaultdict(int)
    for key, var_list in nonphase_dict.items():
        mole_num = int(key)
        count_ = 0
        for one_var in var_list:
            var_pos = int(one_var.split(":")[0])
            var_hp = int(one_var.split(":")[1])
            merge_dict[(var_pos, var_hp)].append(mole_num)
            if count_ == 0:
                var_pos_min = var_pos
            else:
                if var_pos_min > var_pos:
                    var_pos_min = var_pos
            count_ += 1
        mole_var_pos_min_dict[mole_num] = var_pos_min

    phase_block = defaultdict(lambda: defaultdict(int))
    count_phase = defaultdict(int)
    use_key = defaultdict(int)
    for key, mole_list in merge_dict.items():
        if use_key[key] == 0:
            set_PS_tag = key[0]
            key_complement = (key[0],get_complement_hp_2(key[1]))
            for mole_num in mole_list:
                phase_block[mole_num][(set_PS_tag,"hp1")] += 1
                count_phase[(set_PS_tag,"hp1")] += 1
            if key_complement in merge_dict:
                use_key[key_complement] = 1
                mole_list_complement = merge_dict[key_complement]
                for mole_num in mole_list_complement:
                    phase_block[mole_num][(set_PS_tag,"hp2")] += 1
                    count_phase[(set_PS_tag,"hp2")] += 1


    phase_block_use = defaultdict(list)
    for mole_num, phase_info_all in phase_block.items():
        use_phase_info = ["",""]
        curr_ = 0
        for phase_info,val in phase_info_all.items():
            count_1 = count_phase[phase_info]
            #phase_info_complement = (phase_info[0],get_complement_hp_3(phase_info[1]))
            #count_2 = count_phase[phase_info_complement]
            #count_diff = abs(count_1 - count_2)
            count_diff = count_1
            if curr_ == 0:
                count_diff_max = count_diff
                use_phase_info = phase_info
            else:
                if count_diff_max < count_diff:
                    count_diff_max = count_diff
                    use_phase_info = phase_info

            curr_ += 1
        use_PS_tag = use_phase_info[0]
        use_HP_tag = use_phase_info[1]
        phase_block_use[mole_num] = [use_PS_tag, use_HP_tag] 

    save_min_pos = defaultdict(list)
    save_min_pos_2 = defaultdict(int)
    for mole_num, val in phase_block_use.items():
        min_pos = mole_var_pos_min_dict[mole_num]
        save_min_pos[(val[0],val[1])].append(min_pos)

    for key, val in save_min_pos.items():
        save_min_pos_2[key] = np.min(val)

    phase_block_final = defaultdict(list)
    for mole_num, val in phase_block_use.items():
        use_HP_tag = val[1]
        use_PS_tag = save_min_pos_2[(val[0],val[1])]
        phase_block_final[mole_num]= [use_PS_tag, use_HP_tag]

    
    write_phase_block_into_h5(phase_block_final, phase_file_1,output_file_1)
    write_phase_block_into_h5(phase_block_final, phase_file_2,output_file_2)





#!/usr/bin/env python
import pdb
#pdb.set_trace()
from subprocess import Popen,PIPE,STDOUT
from argparse import ArgumentParser
import os
import sys
from multiprocessing import Pool,cpu_count,active_children,Manager
import time
from collections import defaultdict
import pickle
import numpy as np
from Merge_reads_for_PB_of_merged_libs_v2 import *


script_path = os.path.dirname(os.path.abspath( __file__ ))
code_path = script_path + "/" 
__author__ = "Xin Zhou@Stanford"
parser = ArgumentParser(description="Author: xzhou15@cs.stanford.edu\n",usage='use "python3 %(prog)s --help" for more information')
parser.add_argument('--bam_file_list','-bam',help="Required parameter, BAM file list, each BAM file is seperately by comma \",\". For example: 1.bam,2.bam" ,required=True)
parser.add_argument('--vcf_file_list','-v',help="Required parameter, VCF file list, each VCF file is seperately by comma \",\" . For example: 1.vcf,2.vcf",required=True)
parser.add_argument('--chr_start','-start',type=int,help="chromosome start from, default = 1", default=1)
parser.add_argument('--chr_end','-end',type=int,help="chromosome end by, default = 23", default=23)
parser.add_argument('--sample_name_list','-name',help="Required parameter, sample name list, each sample name is seperately by comma \",\". For example: S24385_lysis_2,S24385_lysis_2H",required=True)
parser.add_argument('--out_dir','-o', help="Directory to store Aquila assembly results, default = ./Assembly_results",default="./Asssembly_results")
parser.add_argument('--uniq_map_dir','-uniq_dir', help="Required parameter, Directory for 100-mer uniqness, you can run ./install.sh to download \"Uniqness_map\" for GRCh38",required=True)
parser.add_argument('--num_threads','-t_chr',type=int,help="number of threads, default = 8 (recommended)", default=8)
parser.add_argument('--num_threads_for_bwa_mem','-t',type=int,help="number of threads for bwa-mem, default = 20", default=20)
parser.add_argument('--block_threshold','-bt',type=int,help="phase block threshold, default = 200000(200kb)",default=200000)
parser.add_argument('--block_len_use','-bl',type=int,help="phase block len threshold, default = 100000(100kb)",default=100000)

args = parser.parse_args()
bam_list = [item for item in args.bam_file_list.split(',')]
vcf_list = [item for item in args.vcf_file_list.split(',')]
sample_list = [item for item in args.sample_name_list.split(',')]

def Get_fragment_files(bam_file,vcf_file,chr_start,chr_end,h5_dir):
    use_cmd = "python3 " + code_path + "Run_h5_all_multithreads.py" + " --bam_file " + bam_file + " --vcf_file " + vcf_file + " --sample_name " + sample_name + " --chr_start " + str(chr_start) + " --chr_end " + str(chr_end) + " --mbq 13 --mmq 20 --boundary 50000 " + " --num_threads " + str(num_threads) + " --out_dir " + h5_dir
    Popen(use_cmd,shell=True).wait()


def Get_highconf_profile(bam_file,chr_start,chr_end,HighConf_file_dir,uniq_map_dir):
    use_cmd = "python3 " + code_path + "Generate_highconf_cut_profile_v2.py" + " --bam_file " +  bam_file + " --chr_start " + str(chr_start) + " --chr_end " + str(chr_end) + " --mbq 13 --mmq 20 " + " --out_dir " + HighConf_file_dir + " --uniq_map_dir " + uniq_map_dir
    Popen(use_cmd,shell=True).wait()
  

def Haplotying_fragments(chr_start,chr_end,phased_file_dir,h5_dir,sample_name):
    use_cmd = "python3 " + code_path + "Run_phase_alg_multithreads2_hybrid.py" + " --chr_start " + str(chr_start) + " --chr_end " + str(chr_end) + " --overlap_threshold 3 --support_threshold 5 " + " --out_dir " + phased_file_dir  + " --h5_dir " + h5_dir + " --sample_name " + sample_name
    Popen(use_cmd,shell=True).wait()


def Cut_phase_blocks(chr_start,chr_end,block_threshold,block_len_use,phase_blocks_cut_highconf_dir,phased_file_dir,HighConf_file_dir):
    use_cmd = "python3 " + code_path + "Cut_phaseblock_for_phased_h5_v4.0_highconf_v2.py" + " --chr_start " + str(chr_start) + " --chr_end " + str(chr_end) + " --block_threshold " + str(block_threshold) + " --block_len_use " + str(block_len_use)  + " --out_dir " +  phase_blocks_cut_highconf_dir + " --in_dir " + phased_file_dir + " --highconf_profile_dir " + HighConf_file_dir
    Popen(use_cmd,shell=True).wait()


def Get_fastq_files_total(bam_file,chr_start,chr_end,num_threads,Raw_fastqs_dir,Sorted_bam_dir,xin):
    use_cmd = "python3 " + code_path + "Read_fastqs_from_sortedbam_v2.py " + " --num_threads " + str(num_threads) + " --chr_start " + str(chr_start) + " --chr_end " + str(chr_end) + " --out_dir " + Raw_fastqs_dir + " --bam_file " + bam_file + " --bam_dir " + Sorted_bam_dir
    Popen(use_cmd,shell=True).wait()


def Extract_reads_for_small_chunks(chr_start,chr_end,h5_dir,phase_blocks_cut_highconf_dir,Local_Assembly_dir,Raw_fastqs_dir,block_len_use,sample_name,num_threads,mole_len_dict_prev_file,mole_len_dict_file):
    use_cmd = "python3 " + code_path + "Run_extract_reads_for_smallchunks_all_lessmem_hybrid.py" + " --phase_cut_folder " + phase_blocks_cut_highconf_dir + " --chr_start " + str(chr_start) + " --chr_end " + str(chr_end) + " --out_dir " + Local_Assembly_dir + " --h5_folder " + h5_dir +  " --fastq_folder " + Raw_fastqs_dir + " --cut_threshold " + str(block_len_use) + " --sample_name " + sample_name + " --num_threads " +   str(num_threads) + " --mole_len_dict_prev_file " + mole_len_dict_prev_file + " --mole_len_dict_file " + mole_len_dict_file 
    Popen(use_cmd,shell=True).wait()


def merge_h5_file(h5_file,fw,count):
    f = open(h5_file,"r")
    for line in f:
        data = line.rsplit()
        data[6] = str(count)
        count += 1
        fw.writelines("\t".join(data) + "\n")
    f.close() 
    return count


def extract_bam_by_chr(bam_file,chr_num,sample_bam_chr,xin):
    try:
        extract_bam_chr_cmd = "samtools view -b " + bam_file + " chr" + str(chr_num) + " > " + sample_bam_chr 
    except:
        extract_bam_chr_cmd = code_path + "samtools/" + "samtools view -b " + bam_file + " chr" + str(chr_num) + " > " + sample_bam_chr 

    Popen(extract_bam_chr_cmd,shell=True).wait()


def check_mole_len(cur_mole_file_prefix,chr_start,chr_end):
    mole_len_dict = defaultdict(int)
    for chr_num in range(chr_start,chr_end + 1): 
        cur_mole_file = cur_mole_file_prefix + "_chr" + str(chr_num)
        cat_cmd = "cat " + cur_mole_file + " | wc -l "
        p = Popen(cat_cmd,shell=True,stdout=PIPE, stderr=STDOUT)
        stdout, stderr = p.communicate()
        mole_len_dict[chr_num] = int(stdout.decode().rsplit()[0])
    return mole_len_dict


def add_two_dicts(mole_len_dict_prev,mole_len_dict):
    mole_len_dict_old = mole_len_dict_prev.copy()
    for key, val in mole_len_dict.items():
        mole_len_dict_prev[key] = val + mole_len_dict_old[key]
    return mole_len_dict_prev




if __name__ == "__main__":
    if len(sys.argv) == 1:
        Popen("python3 " + "Aquila_step1.py -h",shell=True).wait()
    else:
        chr_start = args.chr_start
        chr_end = args.chr_end
        block_len_use = args.block_len_use
        block_threshold = args.block_threshold
        uniq_map_dir = args.uniq_map_dir + "/"
        num_threads = args.num_threads
        num_threads_for_bwa_mem = args.num_threads_for_bwa_mem
        HighConf_file_dir = args.out_dir + "/HighConf_file/"
        phased_file_dir = args.out_dir + "/results_phased_probmodel/"
        phase_blocks_cut_highconf_dir = args.out_dir + "/phase_blocks_cut_highconf/"
        Raw_fastqs_dir = args.out_dir + "/Raw_fastqs/"
        Local_Assembly_dir = args.out_dir + "/Local_Assembly_by_chunks/"
        merged_h5_dir = args.out_dir + "/H5_for_molecules/"
        if os.path.exists(merged_h5_dir):
            print("using existing output folder: " + merged_h5_dir)
        else:
            os.makedirs(merged_h5_dir)
        if os.path.exists(HighConf_file_dir):
            print("using existing output folder: " + HighConf_file_dir)
        else:
            os.makedirs(HighConf_file_dir)
        if os.path.exists(Local_Assembly_dir):
            print("using existing output folder: " + Local_Assembly_dir)
        else:
            os.makedirs(Local_Assembly_dir)


        #### Get fragment file and Merge for multi-libs ####
        _num = 0
        h5_file_prefix_list = []
        for _num in range(len(bam_list)):
            bam_file = bam_list[_num]
            vcf_file = vcf_list[_num]
            sample_name = sample_list[_num]
            _num += 1
            h5_dir = args.out_dir + "/H5_for_molecules_" + sample_name + "/"
            Get_fragment_files(bam_file,vcf_file,chr_start,chr_end,h5_dir)
            h5_file_prefix_list.append(h5_dir + sample_name + "_chr")

        for chr_num in range(chr_start,chr_end + 1):
            merged_h5_file = merged_h5_dir + "merged_chr" + str(chr_num)
            merged_h5_file_sorted = merged_h5_dir + "merged_chr" + str(chr_num) + "_sorted.h5"
            fw = open(merged_h5_file,"w")
            count =  1
            for one_file_prefix in h5_file_prefix_list:
                one_file = one_file_prefix + str(chr_num)
                count = merge_h5_file(one_file,fw,count)
            fw.close()
            sorted_cmd = "cat " + merged_h5_file + " | sort -k2n > " + merged_h5_file_sorted 
            print(sorted_cmd)
            Popen(sorted_cmd,shell=True).wait()
        
        
        #### HighConf file ####
        for chr_num in range(chr_start,chr_end + 1):
            merge_bam_chr = HighConf_file_dir + "merged_chr" + str(chr_num) + ".bam"  
            try:
                merge_bam_cmd = "samtools merge " + merge_bam_chr + " "
            except:
                merge_bam_cmd = code_path + "samtools/" + "samtools merge " + merge_bam_chr + " "

            _num = 0
            pool = Pool(len(bam_list))
            for _num in range(len(bam_list)):
                bam_file = bam_list[_num]
                sample_name = sample_list[_num]
                sample_bam_chr = HighConf_file_dir + sample_name + "_chr" + str(chr_num) + ".bam" 
                pool.apply_async(extract_bam_by_chr,(bam_file,chr_num,sample_bam_chr,"xin"))
                merge_bam_cmd += sample_bam_chr + " "
                _num += 1
            pool.close()
            while len(active_children()) > 1:
                time.sleep(0.5)
            pool.join()

            Popen(merge_bam_cmd,shell=True).wait()
            try:
                indx_bam_chr_cmd = "samtools index " + merge_bam_chr
            except:
                indx_bam_chr_cmd = code_path + "samtools/" + "samtools index " + merge_bam_chr

            Popen(indx_bam_chr_cmd,shell=True).wait()
        
        for chr_num in range(chr_start,chr_end + 1):
            merge_bam_chr = HighConf_file_dir + "merged_chr" + str(chr_num) + ".bam"  
            Get_highconf_profile(merge_bam_chr,chr_num,chr_num,HighConf_file_dir,uniq_map_dir)

        #### Haplotying,Cuting PBs ####
        _num = 0
        all_depth_median = []
        for _num in range(len(bam_list)):
            sample_name = sample_list[_num]
            _num += 1
            h5_dir_sample_depth = args.out_dir + "/H5_for_molecules_" + sample_name + "/median_depth_for_var.txt"
            depth_file = open(h5_dir_sample_depth, 'r')
            depth_median = depth_file.readline()
            depth_file.close()
            all_depth_median.append(float(depth_median.rsplit()[0]))
            _num += 1
        depth_avg_libs = np.mean(all_depth_median)
        fw = open(merged_h5_dir + "median_depth_for_var.txt","w")
        fw.write(str(depth_avg_libs) + "\n")
        fw.close()

        sample_name = "merged"
        Haplotying_fragments(chr_start,chr_end,phased_file_dir,merged_h5_dir,sample_name)
        Cut_phase_blocks(chr_start,chr_end,block_threshold,block_len_use,phase_blocks_cut_highconf_dir,phased_file_dir,HighConf_file_dir)

        #### Get fastqs ####
        _num = 0
        pool = Pool(len(bam_list))
        for _num in range(len(bam_list)):
            bam_file = bam_list[_num]
            sample_name = sample_list[_num]
            Raw_fastqs_dir_sample = args.out_dir + "/Raw_fastqs_" + sample_name + "/"
            Bam_dir_sample = args.out_dir + "/sorted_bam_" + sample_name + "/"
            pool.apply_async(Get_fastq_files_total,(bam_file,chr_start,chr_end,num_threads_for_bwa_mem,Raw_fastqs_dir_sample,Bam_dir_sample,"xin"))
            _num += 1
        pool.close()
        while len(active_children()) > 1:
            time.sleep(0.5)
        pool.join()

        #### Extract for small chunks ####
        _num = 0
        mole_len_dict_prev = defaultdict(int)
        for chr_num in range(chr_start,chr_end + 1):
            mole_len_dict_prev[chr_num] = 0
        mole_len_dict_prev_file = args.out_dir + "/mole_len_dict_prev.p"
        pickle.dump(mole_len_dict_prev,open(mole_len_dict_prev_file,"wb"))
        for _num in range(len(bam_list)):
            bam_file = bam_list[_num]
            sample_name = sample_list[_num]
            Raw_fastqs_dir_sample = args.out_dir + "/Raw_fastqs_" + sample_name + "/"
            h5_dir_sample = args.out_dir + "/H5_for_molecules_" + sample_name + "/"
            Local_Assembly_dir_sample = args.out_dir + "/Local_Assembly_by_chunks_" + sample_name  + "/"
            mole_len_dict = check_mole_len(h5_dir_sample + sample_name,chr_start,chr_end) 
            mole_len_dict_file = args.out_dir +  "/mole_len_dict.p"
            pickle.dump(mole_len_dict,open(mole_len_dict_file,"wb"))
            Extract_reads_for_small_chunks(chr_start,chr_end,h5_dir_sample,phase_blocks_cut_highconf_dir,Local_Assembly_dir_sample,Raw_fastqs_dir_sample,block_len_use,sample_name,12,mole_len_dict_prev_file,mole_len_dict_file)
            mole_len_dict_prev = add_two_dicts(mole_len_dict_prev,mole_len_dict)
            pickle.dump(mole_len_dict_prev,open(mole_len_dict_prev_file,"wb"))
            _num += 1


        #### Merge ####
        pool = Pool(chr_end - chr_start + 1)
        for chr_num in range(chr_start,chr_end + 1):
            _num = 0
            in_dir_list = ""
            local_assembly_dir_chr = Local_Assembly_dir + "chr" + str(chr_num) +  "_files_cutPBHC/" 
            for _num in range(len(bam_list)):
                sample_name = sample_list[_num]
                in_dir = args.out_dir + "/Local_Assembly_by_chunks_" + sample_name  + "/"  + "chr" + str(chr_num) +  "_files_cutPBHC/" + ","
                in_dir_list += in_dir
                _num += 1
            pool.apply_async(Merge_reads_for_PB_of_merged_libs(in_dir_list,local_assembly_dir_chr,"xin"))
        pool.close()
        while len(active_children()) > 1:
            time.sleep(0.5)
        pool.join()








import pdb
#pdb.set_trace()
import pickle
import gzip
import glob
from collections import defaultdict
import numpy as np
from subprocess import Popen
from multiprocessing import Pool,cpu_count,active_children
import time
import os
import sys
from argparse import ArgumentParser
parser = ArgumentParser(description="Author: xzhou15@cs.stanford.edu\n",usage='use "python3 %(prog)s --help" for more information')
parser.add_argument('--assembly_vcf','-v_assembly',help="Required parameter, Aquila assembly based VCF file, \"Aquila_final_sorted.vcf\", called by \"Aquila_assembly_based_variants_call.py\"",required=True)
parser.add_argument('--vcf_file','-v',help="Required parameter, VCF file called by FreeBayes",required=True)
parser.add_argument('--sample_name','-sample',help="sample name, you can define your own,for example: L1, default = sample_name",default="sample_name")
parser.add_argument('--chr_start','-start',type=int,help="chromosome start from, default = 1", default=1)
parser.add_argument('--chr_end','-end',type=int,help="chromosome end by, default = 23", default=23)
parser.add_argument('--out_dir','-o', help="Directory to store output, default = ./Aquila_Phasing_Results",default="./Aquila_Phasing_Results")
parser.add_argument('--assembly_dir','-a_dir', help="Required parameter, folder to store Aquila assembly results at Aquila assembly steps",required=True)
parser.add_argument('--block_len_use','-bl',type=int,help="phase block len threshold, default = 100000 (100kb), if you change it, make sure it's consistent with what you define at Aquila assembly steps",default=100000)
args = parser.parse_args()
__author__ = "Xin Zhou@Stanford"


# 1. Phasing assembled variants by contigs and phase blocks
def check_PS_flag_in_HCbk(HCbk_dict,PS_flag_1,PS_flag_2):
    flag = 0
    flag_1 = 0
    flag_2 = 0
    for key,val in HCbk_dict.items():
        flag_1 = int(key[0])
        flag_2 = int(key[1])
        if PS_flag_1 >= flag_1 and PS_flag_2 <= flag_2:
            flag = 1
            break

    return (flag,flag_1,flag_2)


def Phasing_assembled_SV(HCbk_dict,vcf_file,chr_num,out_dir):
    phased_variants = defaultdict(lambda: defaultdict(list))
    phased_variants_loci = defaultdict(lambda: defaultdict(list))
    #f = gzip.open(vcf_file,"rb")
    f = open(vcf_file,"rb")
    if chr_num == 23:
        chr_num = "X"
    for line in f:
        data = line.decode().rsplit()
        if data[0][0] != "#":
            chr_num_curr = data[0]
            if chr_num_curr == "chr" + str(chr_num):
                GT = data[9].split(":")[0]
                if GT == "0/1":
                    if int(data[9].split(":")[1].split("_")[0]) == 0:
                        PS_info = data[9].split("PS")[2].split("_")
                        PS_flag_1 = int(PS_info[0])
                        PS_flag_2 = int(PS_info[1])
                        hp_flag = PS_info[2]
                    else:
                        PS_info = data[9].split("PS")[1].split("_")
                        PS_flag_1 = int(PS_info[0])
                        PS_flag_2 = int(PS_info[1])
                        hp_flag = PS_info[2]
                    flag,flag_1,flag_2 = check_PS_flag_in_HCbk(HCbk_dict,PS_flag_1,PS_flag_2)
                    if flag == 1:
                        phased_variants[(flag_1,flag_2)][hp_flag].append(data)
                        phased_variants_loci[(flag_1,flag_2)][hp_flag].append(int(data[1]))
                    else:
                        phased_variants[(PS_flag_1,PS_flag_2)][hp_flag].append(data)
                        phased_variants_loci[(PS_flag_1,PS_flag_2)][hp_flag].append(int(data[1]))
                        #print(PS_flag_2 - PS_flag_1)

    count = 0
    count_2 = 0
    total_len = 0
    PS_dict = defaultdict(list)
    for key,val in phased_variants_loci.items():
        if len(val.keys()) == 2:
            count += 1
            min_loci_hp1 = min(val['hp1'])
            max_loci_hp1 = max(val['hp1'])
            min_loci_hp2 = min(val['hp2'])
            max_loci_hp2 = max(val['hp2'])
            min_loci = min([min_loci_hp1,min_loci_hp2])
            max_loci = max([max_loci_hp1,max_loci_hp2])
            PS_dict[key] = [min_loci,max_loci]
        else:
            count_2 += 1
            for key_2,val_2 in val.items():
                min_loci = min(val_2)
                max_loci = max(val_2)
                PS_dict[key] = [min_loci,max_loci]

    ### write to phased.vcf
    if chr_num == "X":
        chr_num = 23
    fw = open(out_dir + "phased_chr" + str(chr_num) + ".vcf","w")
    for key,val in phased_variants.items():
        print(key,key[1] - key[0])
        PS_info = PS_dict[key]
        PS_start = PS_info[0]
        PS_end = PS_info[1]
        for key_2, val_2 in val.items():
            if key_2 == "hp1":
                for data in val_2:
                    data[8] = "GT:PS"
                    data[9] = "1|0" + ":" + str(PS_start) + "-" + str(PS_end)
                    fw.write("\t".join(data) + "\n")
            elif key_2 == "hp2":
                for data in val_2:
                    data[8] = "GT:PS"
                    data[9] = "0|1" + ":" + str(PS_start) + "-" + str(PS_end)
                    fw.write("\t".join(data) + "\n")

    
    fw.close()
 

# 2. Phasing SNPs by Haplotyping Alg
def Phasing_SNPs_byAlg(in_dir,chr_num,SNP_dict,out_dir):
    fw = open(out_dir + "chr" + str(chr_num) + "_byAlg.vcf","w")
    # phasing blocks from haplotyping algorithms
    all_files = glob.glob(in_dir + "/" +  "chr" + str(chr_num) + "_*_final.p")
    all_pos = []
    total_num = 0
    for one_file in all_files:
        all_blocks = pickle.load(open(one_file,"rb"))
        total_num += len(all_blocks)
        for _num in range(len(all_blocks)):
            _block = all_blocks[_num]
            sorted_pos = sorted(_block.keys())
            _start_PS = np.min(sorted_pos)
            _end_PS = np.max(sorted_pos)
            print(_start_PS)
            for _pos, val in _block.items():
                all_pos.append(_pos)
                _key = (_pos + 1, val[1], val[2])
                if _key in SNP_dict:
                    data = SNP_dict[_key]
                    if val[0] == 2:
                        data[8] = data[8] + ":PS"
                        temp = data[9].split(":")
                        temp[0] = "1|0"
                        temp.append(str(_start_PS))
                        data[9] = ":".join(temp)
                        fw.write("\t".join(data) + "\n")
                    if val[0] == 1:
                        data[8] = data[8] + ":PS"
                        temp = data[9].split(":")
                        temp[0] = "0|1"
                        temp.append(str(_start_PS))
                        data[9] = ":".join(temp)
                        fw.write("\t".join(data) + "\n")
                    del SNP_dict[_key] 
                else:
                    print(_pos,val)

    fw.close()

    return len(all_pos)


def read_vcf_into_dict(vcf_file,chr_num,out_dir):
    if chr_num == 23:
        chr_num = "X"
    SNP_dict = defaultdict()
    with open(vcf_file) as f:
        for line in f:
            data = line.rsplit()
            if data[0][0] != "#":
                chr_num_curr = data[0]
                if chr_num_curr == "chr" + str(chr_num):
                    _pos = int(data[1])
                    ref = data[3]
                    alt = data[4]
                    GT = data[9].split(":")[0]
                    if (GT == "0/1" or GT == "1/0") and len(ref) == 1 and len(alt) == 1:
                    #if (GT == "0/1" or GT == "1/0"):
                        SNP_dict[(_pos,ref,alt)] = data
    if chr_num == "X":
        chr_num = 23
    pickle.dump(SNP_dict, open(out_dir + "chr" + str(chr_num) + "_SNPs_dict.p","wb"))
    return SNP_dict


# 3. Impute to phase assembled variants by SNPs(phased by Alg) 
def reverse_phase(phase_tag):
    if phase_tag == "0|1":
        return "1|0"
    elif phase_tag == "1|0":
        return "0|1"


def Compare_two_sets(phased_vcf,assembled_vcf,output_file,out_dir,chr_num,xin):
    temp_file = out_dir + "/_tmp_chr" + str(chr_num) + ".vcf"
    fw = open(temp_file,"w")
    dict_1 = defaultdict(list)
    dict_2 = defaultdict(list)
    with open(assembled_vcf,"r") as f:
        for line in f:
            data = line.rsplit()
            _pos = int(data[1]) + 1
            dict_1[_pos] = data
            
    with open(phased_vcf,"r") as f:
        for line in f:
            data = line.rsplit()
            _pos = int(data[1]) 
            dict_2[_pos] = data

    shared_loci = set(dict_1.keys()) & set(dict_2.keys())
    uniq_1 = list(set(dict_1.keys()) - shared_loci)
    uniq_2 = list(set(dict_2.keys()) - shared_loci)
    shared_loci_list = list(shared_loci)
    count = 0
    count_1 = 0
    count_wrong = 0
    count_miss_1 = 0
    count_miss_2 = 0
    for _pos in uniq_1:
        try:
            total_pos = shared_loci_list + [_pos]
            total_pos_sorted = sorted(total_pos)
            idx = total_pos_sorted.index(_pos)
            _pos_before = total_pos_sorted[idx - 1]
            _pos_after = total_pos_sorted[idx + 1]

            before_info_1 = dict_1[_pos_before][9].split(":")
            after_info_1 = dict_1[_pos_after][9].split(":")
            _info_1 = dict_1[_pos][9].split(":")
            
            before_info_2 = dict_2[_pos_before][9].split(":")
            after_info_2 = dict_2[_pos_after][9].split(":")

            if before_info_1[1] == after_info_1[1] and before_info_1[1] == _info_1[1] and before_info_2[-1] == after_info_2[-1]:
                before_phase_1 = before_info_1[0]
                after_phase_1 = after_info_1[0]
                _phase_1 = _info_1[0]
                before_phase_2 = before_info_2[0]
                after_phase_2 = after_info_2[0]
                if before_phase_1 == before_phase_2:
                    if after_phase_1 == after_phase_2:
                        count += 1
                        use_info = dict_1[_pos]
                        use_info[9] = _phase_1 + ":"  + before_info_2[-1]
                        use_info[1] =str(int(use_info[1]) + 1)   # + 1 for locus 
                        fw.write("\t".join(use_info) + "\n")
                    else:
                        count_wrong += 1
                if before_phase_1 != before_phase_2:
                    if after_phase_1 != after_phase_2:
                        count += 1
                        use_info = dict_1[_pos]
                        use_info[9] = reverse_phase(_phase_1) + ":"  + before_info_2[-1]
                        use_info[1] =str(int(use_info[1]) + 1)
                        fw.write("\t".join(use_info) + "\n")
                    else:
                        count_wrong += 1
            elif before_info_1[1] == _info_1[1]:
                count_1 += 1
                before_phase_1 = before_info_1[0]
                _phase_1 = _info_1[0]
                before_phase_2 = before_info_2[0]
                if before_phase_1 == before_phase_2:
                    use_info = dict_1[_pos]
                    use_info[9] = _phase_1 + ":"  + before_info_2[-1]
                    use_info[1] =str(int(use_info[1]) + 1)
                    fw.write("\t".join(use_info) + "\n")
                if before_phase_1 != before_phase_2:
                    use_info = dict_1[_pos]
                    use_info[9] = reverse_phase(_phase_1) + ":"  + before_info_2[-1]
                    use_info[1] =str(int(use_info[1]) + 1)
                    fw.write("\t".join(use_info) + "\n")
            elif after_info_1[1] == _info_1[1]:
                count_1 += 1
                after_phase_1 = after_info_1[0]
                _phase_1 = _info_1[0]
                after_phase_2 = after_info_2[0]
                if after_phase_1 == after_phase_2:
                    use_info = dict_1[_pos]
                    use_info[9] = _phase_1 + ":"  + after_info_2[-1]
                    use_info[1] =str(int(use_info[1]) + 1)
                    fw.write("\t".join(use_info) + "\n")
                if after_phase_1 != after_phase_2:
                    use_info = dict_1[_pos]
                    use_info[9] = reverse_phase(_phase_1) + ":"  + after_info_2[-1]
                    use_info[1] =str(int(use_info[1]) + 1)
                    fw.write("\t".join(use_info) + "\n")
            else:
                count_miss_1 += 1
        except:
            count_miss_2 += 1
            pass
    print("------------result for chr" + str(chr_num) + ": ")
    print(len(uniq_1))
    print(count,count_1,count_wrong)
    print(count_miss_1,count_miss_2)
    fw.close()
    correct_percent = float(count + count_1)/(count + count_1 + count_wrong)
    print(correct_percent)

    fw = open(output_file,"w")
    with open(temp_file,"r") as f:
        for line in f:
            data = line.rsplit()
            ref = data[3]
            alt = data[4]
            if ref == "-" and len(alt) == 1:
                data[7] = "SVTYPE=INS"
            elif alt == "-" and len(ref) == 1:
                data[7] = "SVTYPE=DEL"
            fw.write("\t".join(data) + "\n")

    fw.close()                


    return correct_percent


def get_homozygous_variants(assembly_vcf,output_file,all_impute_vcf,all_byAlg_vcf,freebayes_vcf):
    fw = open(output_file,"w")
    snp_dict = defaultdict(int)
    with open(all_impute_vcf,"r") as f:
        for line in f:
            data = line.rsplit()
            chr_num = data[0]
            locus = int(data[1])
            snp_dict[(chr_num,locus)] = 1

    with open(all_byAlg_vcf,"r") as f:
        for line in f:
            data = line.rsplit()
            chr_num = data[0]
            locus = int(data[1])
            snp_dict[(chr_num,locus)] = 1

    with open(freebayes_vcf,"r") as f:
        for line in f:
            data = line.rsplit()
            if data[0][0] != "#":
                chr_num = data[0]
                locus = int(data[1])
                ref = data[3]
                alt = data[4]
                GT = data[-1].split(":")[0]
                if len(ref) == 1 and len(alt) == 1 and (GT == "1/1" or GT == "1|1"):
                    if (chr_num,locus) not in snp_dict:
                        snp_dict[(chr_num,locus)] = 1
                        data[8] = "GT"
                        data[9] = "1|1"
                        data[1] = str(locus)
                        fw.write("\t".join(data) + "\n")

    with open(assembly_vcf,"r") as f:
        for line in f:
            data = line.rsplit()
            if data[0][0] != "#":
                GT = data[9].split(":")[0]
                if GT == "1|1" or GT == "1/1":
                    chr_num = data[0]
                    locus = int(data[1]) + 1
                    ref = data[3]
                    alt = data[4]
                    if ref == "-" or alt == "-":   # indel or sv, use all homo
                        data[8] = "GT"
                        data[9] = "1|1"
                        data[1] = str(locus)
                        fw.write("\t".join(data) + "\n")
                    else:
                        if (chr_num,locus) not in snp_dict:
                            data[8] = "GT"
                            data[9] = "1|1"
                            data[1] = str(locus)
                            fw.write("\t".join(data) + "\n")
    fw.close()


def get_unphased_variants(phased_vcf,freebayes_vcf,assembly_vcf,unphased_vcf):
    fw = open(unphased_vcf,"w")
    count = 0
    snp_dict = defaultdict(int)
    indel_dict = defaultdict(int)
    sv_dict = defaultdict(int)
    with open(phased_vcf,"r") as f:
        for line in f:
            data = line.rsplit()
            ref = data[3]
            alt = data[4]
            chr_num = data[0]
            locus = int(data[1])
            if len(ref) == 1 and len(alt) == 1 and ref != "-" and alt != "-":
                snp_dict[(chr_num,locus)] = 1
            elif (ref == "-" and len(alt) < 50) or (alt == "-" and len(ref) < 50):
                indel_dict[(chr_num,locus)] =  1
            elif (ref == "-" and len(alt) >= 50) or (alt == "-" and len(ref) >= 50):
                sv_dict[(chr_num,locus)] =  1

    ################### snp ####################### 
    print("snp count")
    count_1 = 0
    print(len(snp_dict))
    with open(freebayes_vcf,"r") as f:
        for line in f:
            data = line.rsplit()
            if data[0][0] != "#":
                ref = data[3]
                alt = data[4]
                chr_num = data[0]
                locus = int(data[1])
                GT = data[-1].split(":")[0]
                if len(ref) == 1 and len(alt) == 1 and (GT == "0/1" or GT == "1/0"):
                    if (chr_num,locus) not in snp_dict:
                        fw.write(line)
                        count_1 += 1
                        snp_dict[(chr_num,locus)] = 1

    count_2 = 0
    count_3 = 0
    count_4 = 0
    print(len(snp_dict))
    with open(assembly_vcf,"r") as f:
        for line in f:
            data = line.rsplit()
            ref = data[3]
            alt = data[4]
            chr_num = data[0]
            locus = int(data[1]) + 1
            GT = data[-1].split(":")[0]
            if len(ref) == 1 and len(alt) == 1 and ref != "-" and alt != "-" and GT == "0/1": #snp
                if (chr_num,locus) not in snp_dict:
                    data[1] = str(locus)
                    fw.write("\t".join(data) + "\n")
                    count_2 += 1
            elif (len(ref) >= 50 and alt == "-") or (ref == "-" and len(alt) >= 50) and GT == "0/1": # sv
                if (chr_num,locus) not in sv_dict:
                    data[1] = str(locus)
                    fw.write("\t".join(data) + "\n")
                    count_3 += 1
            elif (len(ref) < 50 and alt == "-") or (ref == "-" and len(alt) < 50) and GT == "0/1": # indel
                if (chr_num,locus) not in indel_dict:
                    data[1] = str(locus)
                    fw.write("\t".join(data) + "\n")
                    count_4 += 1
    print("freebayes_snp,assembly_snp,indel,sv:")
    print(count_1,count_2,count_4,count_3)
    fw.close()


def reformat_vcf_1(vcf_file,output_file):
    fw = open(output_file,"w")
    with open(vcf_file,"r") as f:
        for line in f:
            data = line.rsplit()
            GT = data[-1].split(":")[0]
            data[-2] = "GT"
            data[-1] = GT
            data[2] = "."
            data[5] = "."
            data[-4] = "."
            ref = data[3]
            alt = data[4]
            if len(ref) == 1 and len(alt) == 1 and ref != "-" and alt != "-":
                data[7] = "VTYPE=SNP"
            elif ref == "-" and len(alt) > 0:
                data[7] = "VTYPE=INS"
            elif alt == "-" and len(ref) > 0:
                data[7] = "VTYPE=DEL"
            fw.write("\t".join(data) + "\n")
    fw.close()


def reformat_vcf_2(vcf_file,output_file):
    fw = open(output_file,"w")
    with open(vcf_file,"r") as f:
        for line in f:
            data = line.rsplit()
            if data[-2] == "GT":
                data[2] = "."
                data[5] = "."
                data[-4] = "."
                ref = data[3]
                alt = data[4]
                if len(ref) == 1 and len(alt) == 1 and ref != "-" and alt != "-":
                    data[7] = "VTYPE=SNP"
                elif ref == "-" and len(alt) > 0:
                    data[7] = "VTYPE=INS"
                elif alt == "-" and len(ref) > 0:
                    data[7] = "VTYPE=DEL"
                fw.write("\t".join(data) + "\n")
            else:
                try:
                    GT_idx = data[-2].split(":").index("GT")
                except:
                    print(data)
                PS_idx = data[-2].split(":").index("PS")
                GT = data[-1].split(":")[GT_idx]
                PS = data[-1].split(":")[PS_idx]
                data[-2] = "GT:PS"
                data[-1] = GT + ":"  + PS
                data[2] = "."
                data[5] = "."
                data[-4] = "."
                ref = data[3]
                alt = data[4]
                if len(ref) == 1 and len(alt) == 1 and ref != "-" and alt != "-":
                    data[7] = "VTYPE=SNP"
                elif ref == "-" and len(alt) > 0:
                    data[7] = "VTYPE=INS"
                elif alt == "-" and len(ref) > 0:
                    data[7] = "VTYPE=DEL"
                fw.write("\t".join(data) + "\n")

    fw.close()


def add_header(header_file,sample_name):
    fw = open(header_file,"w")
    fw.writelines("##fileformat=VCFv4.2\n")
    fw.writelines("##source=Aquila\n")
    fw.writelines("##INFO=<ID=VTYPE,Number=1,Type=String,Description=\"Type of SV:DEL=Deletion, INS=Insertion, SNP=snps\">\n")
    fw.writelines("##FORMAT=<ID=GT,Number=1,Type=String,Description=\"Genotype\">\n")
    fw.writelines("##FORMAT=<ID=PS,Number=1,Type=String,Description=\"Unique Phase Set\">\n")
    fw.writelines("#CHROM" + "\t" + "POS" + "\t" + "ID" + "\t" + "REF" + "\t" + "ALT" + "\t" + "QUAL" + "\t" + "FILTER" + "\t" + "INFO" + "\t" + "FORMAT" + "\t"  + sample_name + "\n")
    fw.close()


def CleanData(out_dir):
    rm_cmd = "rm " + out_dir + "*chr*"
    Popen(rm_cmd,shell=True).wait()
    rm_cmd = "rm " + out_dir + "*_1.vcf*"
    Popen(rm_cmd,shell=True).wait()
    rm_cmd = "rm " + out_dir + "*_2.vcf*"
    Popen(rm_cmd,shell=True).wait()
    rm_cmd = "rm " + out_dir + "header"
    Popen(rm_cmd,shell=True).wait()
    rm_cmd = "rm " + out_dir + "*phased.vcf"
    Popen(rm_cmd,shell=True).wait()
    rm_cmd = "rm " + out_dir + "all_*.vcf"
    Popen(rm_cmd,shell=True).wait()



def main():
    if len(sys.argv) == 1:
        Popen("python3 " + "Aquila_phasing_all_variants.py -h",shell=True).wait()
    else:
        out_dir = args.out_dir + "/"
        if os.path.exists(out_dir):
            print("using existing output folder: " + out_dir)
        else:
            os.makedirs(out_dir)
        chr_start = args.chr_start
        chr_end = args.chr_end
        sample_name = args.sample_name
        assembled_vcf = args.assembly_vcf
        vcf_file = args.vcf_file
        assemble_dir = args.assembly_dir
        phase_blocks_cut_highconf_dir = assemble_dir + "/phase_blocks_cut_highconf/"
        phased_file_dir = assemble_dir + "/results_phased_probmodel/"
        block_len_use = args.block_len_use
        # 1. phasing assembled variants
        for chr_num in range(chr_start,chr_end + 1):
            breakpoint_file = phase_blocks_cut_highconf_dir + "chr" + str(chr_num) + ".phased_final_cut_by_" + str(block_len_use) + "_HC_breakpoint_2.p" 
            HCbk_dict = pickle.load(open(breakpoint_file,"rb"))
            Phasing_assembled_SV(HCbk_dict,assembled_vcf,chr_num,out_dir)
        # 2. phasing SNPs by Alg
        total_num = 0
        total_snp = 0
        for chr_num in range(chr_start,chr_end + 1):
            SNP_dict = read_vcf_into_dict(vcf_file,chr_num,out_dir)
            ##SNP_dict = pickle.load(open(out_dir + "chr" + str(chr_num) + "_SNPs_dict.p","rb"))
            total_snp += len(SNP_dict)

            all_pos = Phasing_SNPs_byAlg(phased_file_dir, chr_num, SNP_dict,out_dir)
            total_num += all_pos
        print("total_num: " + str(total_num))
        print("total_snp: " + str(total_snp))
        # 3. Impute to phasing assembled variants
        pool = Pool(chr_end - chr_start + 1)
        for chr_num in range(chr_start,chr_end + 1):
            output_file = out_dir + "chr" + str(chr_num) + "_impute.vcf"
            phased_vcf = out_dir + "chr" +str(chr_num) + "_byAlg.vcf"
            assembled_vcf_chr = out_dir + "phased_chr" + str(chr_num) + ".vcf"
            pool.apply_async(Compare_two_sets,(phased_vcf,assembled_vcf_chr,output_file,out_dir,chr_num,"xin"))
        pool.close()
        while len(active_children()) > 1:
            time.sleep(0.5)
        pool.join()
        print("done~")

        for chr_num in range(chr_start,chr_end + 1):
            one_impute = out_dir + "chr" + str(chr_num) + "_impute.vcf"
            one_byAlg = out_dir + "chr" + str(chr_num) + "_byAlg.vcf"
            cat_cmd_1 = "cat " + one_impute  + " >> " + out_dir + "all_impute.vcf"
            cat_cmd_2 = "cat " + one_byAlg  + " >> " + out_dir + "all_byAlg.vcf"
            Popen(cat_cmd_1,shell=True).wait()
            Popen(cat_cmd_2,shell=True).wait()
        all_impute_vcf = out_dir + "all_impute.vcf"
        all_byAlg_vcf = out_dir + "all_byAlg.vcf"
        all_homo_vcf = out_dir + "all_homo.vcf"
        get_homozygous_variants(assembled_vcf,all_homo_vcf,all_impute_vcf,all_byAlg_vcf,vcf_file)

        ########### cat all_impute, all_byAlg, all_homo ###########
        phased_vcf = out_dir + "phased.vcf"
        phased_vcf_2 = out_dir + "phased_2.vcf"
        cat_cmd = "cat " + all_impute_vcf + " " + all_byAlg_vcf + " " + all_homo_vcf + " > " + phased_vcf
        Popen(cat_cmd,shell=True).wait()
        ########### extract unphased variants  ###########
        unphased_vcf = out_dir + "unphased.vcf"
        unphased_vcf_2 = out_dir + "unphased_2.vcf"
        get_unphased_variants(phased_vcf,vcf_file,assembled_vcf,unphased_vcf)
        
        header_file = out_dir + "header"
        add_header(header_file,sample_name)

        final_vcf_1 = out_dir + "Aquila_phased_1.vcf"
        reformat_vcf_1(unphased_vcf,unphased_vcf_2) 
        reformat_vcf_2(phased_vcf,phased_vcf_2) 
        cat_cmd = "cat " + phased_vcf_2 + " " + unphased_vcf_2 + " > " + final_vcf_1
        Popen(cat_cmd,shell=True).wait()
        
        sorted_vcf_1 = out_dir + "Aquila_phased_sorted_1.vcf"
        sort_cmd = "cat " + final_vcf_1 + " | awk '$1 ~ /^#/ {print $0;next} {print $0 | \"LC_ALL=C sort -k1,1 -k2,2n\"}' > " + sorted_vcf_1
        Popen(sort_cmd,shell=True).wait()

        final_vcf = out_dir + "Aquila_phased_sorted.vcf"
        cat_cmd = "cat "+ header_file + " " + sorted_vcf_1 + " > " + final_vcf
        Popen(cat_cmd,shell=True).wait()
        CleanData(out_dir)
        

if __name__ == "__main__":
	main()
        



from setuptools import setup, find_packages, Extension

setup(name='aqtest1',
      version='0.3',
      description='this is just a test',
      author='HENRY TEST',
      author_email='liuyichen@std.uestc.edu.cn',
      packages=['aqtest1',],
      install_requires=[
      
      'pysam',
      'numpy',
      'scipy',],
      dependency_links = [
      'https://github.com/samtools/samtools/tarball/master.tar.gz#egg=samtools-1.9',
      'https://github.com/lh3/minimap2/tarball/master.tar.gz#egg=minimap2-2.17',],
      entry_points={'console_scripts':['step1=aqtest.Aquila_step1:main','step2=aqtest.Aquila_step2:main','assemble=aqtest.Aquila_assembly_based_variants_call:main','phasing=aqtest.Aquila_phasing_all_variants:main']},
      zip_safe=False)



import pdb
#pdb.set_trace()
from collections import defaultdict
import pickle
import numpy as np
from argparse import ArgumentParser
import os
import sys

parser = ArgumentParser(description="Variants Calling:")
parser.add_argument('--chr_start','-start',type=int,help="chromosome start from", default=1)
parser.add_argument('--chr_end','-end',type=int,help="chromosome end by", default=22)
parser.add_argument('--out_dir','-o_dir', help="Directory to store outputs", default='results_/')

args = parser.parse_args()


def Extract_SNV_info(hap_file,output_file,chr_num):
    f = open(hap_file,"r")
    contig_dict = defaultdict(list)
    curr = 0
    count_2 = 0
    for line in f:
        #print(curr)
        curr += 1
        data = line.rsplit()
        if data[0] == "V":
            mapq = int(data[5])
            repeat = int(data[4])
            cur_chr_num = data[1]
            if cur_chr_num == "chrX":
                cur_chr_num = "chr23"
            if cur_chr_num == "chr" + str(chr_num): 
                cur_chr_num = data[1]
                ref_start = int(data[2])
                ref_end = int(data[3])
                ref = data[6]
                alt = data[7]
                contig_num = data[8]
                contig_start = int(data[9])
                contig_end = int(data[10])
                strand_dir = data[-1]
                contig_dict[(cur_chr_num,ref_start)].append([cur_chr_num,ref_start, ref_end, ref, alt, contig_num,contig_start,contig_end,strand_dir])

    contig_dict_filter = defaultdict(list)
    count = 0
    for key, val in contig_dict.items():
        #if len(val) > 1:
            #print(">1:")
            #print(val)
            #count += 1
        #else:
        contig_dict_filter[key] = val[0]
    
    SNV_dict = defaultdict(list)
    for key, val in contig_dict_filter.items():
        ref = val[3] 
        alt = val[4]
        #if len(ref) == 1 and len(alt) == 1:
        if len(ref) == 1 and len(alt) == 1 and ref != "-" and alt != "-":    
            SNV_dict[key] = val
        
    pickle.dump(SNV_dict, open(output_file,"wb"))

    return SNV_dict


def Extract_ref_info(hap_file,output_file,chr_num):
    f = open(hap_file,"r")
    contig_dict = defaultdict(list)
    curr = 0
    count_2 = 0
    ref_dict = defaultdict(int)
    for line in f:
        #print(curr)
        curr += 1
        data = line.rsplit()
        if data[0] == "R":
            start_ = int(data[2])
            end_ = int(data[3])
            for _step in range(start_,end_+1):
                ref_dict[_step] = 1

    pickle.dump(ref_dict, open(output_file,"wb"))

    return ref_dict


def compare_two_haploid_SV(SV_dict_contig_1_file, SV_dict_contig_2_file,ref_dict_contig_1_file,ref_dict_contig_2_file,chr_num,out_dir):
    SV_dict_contig_1 = pickle.load(open(SV_dict_contig_1_file,"rb"))
    SV_dict_contig_2 = pickle.load(open(SV_dict_contig_2_file,"rb"))
    ref_dict_contig_1 = pickle.load(open(ref_dict_contig_1_file,"rb"))
    ref_dict_contig_2 = pickle.load(open(ref_dict_contig_2_file,"rb"))
    count = 0
    count_same = 0
    count_same_wrong = 0
    count_diff = 0
    snv_homo = defaultdict(list)
    snv_hetero = defaultdict(list)
    snv_total = defaultdict(list)
    count_hetero_1 = 0
    count_hetero_2 = 0
    for key, val in SV_dict_contig_1.items():
        flag = 0
        if key in SV_dict_contig_2:
            val_2 = SV_dict_contig_2[key]
            chr_num_1 = val[0]
            chr_num_2 = val_2[0]
            start_1 = val[1]
            end_1 = val[2]
            start_2 = val_2[1]
            end_2 = val_2[2]
            ref_1 = val[3]
            ref_2 = val_2[3]
            alt_1 = val[4]
            alt_2 = val_2[4]
            #print(val,val_2)
            if val[:-4] == val_2[:-4]:
               # print(val[-1],val_2[-1])
                PS_start_1 = int(val[-4].split("_PS")[-1].split("_hp1")[0].split(":")[0])
                PS_end_1 = int(val[-4].split("_PS")[-1].split("_hp1")[0].split(":")[1])
                PS_start_2 = int(val_2[-4].split("_PS")[-1].split("_hp2")[0].split(":")[0])
                PS_end_2 = int(val_2[-4].split("_PS")[-1].split("_hp2")[0].split(":")[1])
                if PS_end_1  < PS_start_2 or PS_end_2 < PS_start_1:
                    count_same_wrong += 1
                else:
                    count_same += 1
                    contig_num_1 = val[-4]
                    contig_start_1 = val[-3]
                    contig_end_1 = val[-2]
                    contig_strand_dir_1 = val[-1]
                    contig_num_2 = val_2[-4]
                    contig_start_2 = val_2[-3]
                    contig_end_2 = val_2[-2]
                    contig_strand_dir_2 = val_2[-1]

                    snv_homo[key] = [chr_num_1,start_1,end_1,ref_1,alt_1,contig_num_1,contig_start_1,contig_end_1,contig_strand_dir_1,contig_num_2,contig_start_2,contig_end_2,contig_strand_dir_2]  
                    snv_total[key] = [chr_num_1,start_1,end_1,ref_1,alt_1,contig_num_1,contig_start_1,contig_end_1,contig_strand_dir_1,contig_num_2,contig_start_2,contig_end_2,contig_strand_dir_2]  
            else:
                #print(val,val_2)
                count_diff += 1
            count += 1
        else:
            if key[1] in ref_dict_contig_2:
                start_1 = val[1]
                end_1 = val[2]
                ref_1 = val[3]
                alt_1 = val[4]
                contig_start_1 = val[-3]
                contig_end_1 = val[-2]
                contig_num_1 = val[-4]
                contig_strand_dir_1 = val[-1]
                count_hetero_1 += 1
                snv_hetero[key] = [val[0],start_1,end_1,ref_1,alt_1,contig_num_1,contig_start_1,contig_end_1,contig_strand_dir_1,'0_PS0:0_hp0',0,0,0]
 
                snv_total[key] = [val[0],start_1,end_1,ref_1,alt_1,contig_num_1,contig_start_1,contig_end_1,contig_strand_dir_1,'0_PS0:0_hp0',0,0,0]
 

    for key_2, val_2 in SV_dict_contig_2.items():
        if key_2 not in SV_dict_contig_1:
            if key_2[1] in ref_dict_contig_1:
                start_2 = val_2[1]
                end_2 = val_2[2]
                ref_2 = val_2[3]
                alt_2 = val_2[4]
                contig_num_2 = val_2[-4]
                contig_start_2 = val_2[-3]
                contig_end_2 = val_2[-2]
                contig_strand_dir_2 = val_2[-1]
                count_hetero_2 += 1
                snv_hetero[key_2] = [val_2[0],start_2,end_2,ref_2,alt_2,'0_PS0:0_hp0',0,0,0,contig_num_2,contig_start_2,contig_end_2,contig_strand_dir_2]

 
                snv_total[key_2] = [val_2[0],start_2,end_2,ref_2,alt_2,'0_PS0:0_hp0',0,0,0,contig_num_2,contig_start_2,contig_end_2,contig_strand_dir_2]

 

    count_hetero = len(SV_dict_contig_1) - count
    pickle.dump(snv_homo,open(out_dir + "snv_homo_chr" + str(chr_num) + ".p","wb"))
    pickle.dump(snv_hetero,open(out_dir + "snv_hetero_chr" + str(chr_num) + ".p","wb"))
    pickle.dump(snv_total,open(out_dir + "snv_total_chr" + str(chr_num) + ".p","wb"))
    return (snv_homo,snv_hetero,count_same,count_hetero,count_same_wrong,count_hetero_1,count_hetero_2)


def extract_SV(lib_file,lib_prefix,indel_tag,h_tag):
    lib_dict = pickle.load(open(lib_file,"rb"))
    fw_lib_contig1 = open(lib_prefix + "_contig1.bed","w") 
    fw_lib_contig2 = open(lib_prefix + "_contig2.bed","w") 
    fw_lib_ref = open(lib_prefix + "_ref.bed","w") 
    count = 1
    for key, val in lib_dict.items():
        fw_lib_contig1.writelines(str(val[3]) + "\t" + str(val[4]) + "\t" + str(val[5]) + "\t" + indel_tag + "\t" + h_tag+ "\t" + str(count) + "\n")
        fw_lib_contig2.writelines(str(val[6]) + "\t" + str(val[7]) + "\t" + str(val[8]) + "\t" + indel_tag + "\t" + h_tag + "\t" + str(count)+ "\n")
        fw_lib_ref.writelines("chr" + str(val[0]) + "\t" + str(val[1]) + "\t" + str(val[2]) + "\t" + indel_tag + "\t" + h_tag + "\t" + str(count) + "\n")
        count += 1


def Write_vcf_for_SV(hetero_file,homo_file,fw,count_id):
    homo = pickle.load(open(homo_file,"rb"))
    hetero = pickle.load(open(hetero_file,"rb"))
    for key, val in homo.items():
        chr_num = str(val[0])
        start_ = val[1]
        end_ = val[2]
        ref = val[3]
        alt = val[4]
        GT = "1/1"
        contig_num_1 = val[5]
        contig_start_1 = val[6]
        contig_end_1 = val[7]
        contig_strand_dir_1 = val[8]
        contig_num_2 = val[9]
        contig_start_2 = val[10]
        contig_end_2 = val[11]
        contig_strand_dir_2 = val[12]
        new_contig_num_1 = contig_num_1.replace(":","_")
        new_contig_num_2 = contig_num_2.replace(":","_")
        contig_info = str(new_contig_num_1) + "_" + str(contig_start_1) + "_" + str(contig_end_1) + "_" + contig_strand_dir_1 + "_" +  str(new_contig_num_2) + "_" + str(contig_start_2) + "_" + str(contig_end_2) + "_" + contig_strand_dir_2 

        fw.writelines(chr_num + "\t" + str(start_) + "\t" + "event" + str(count_id) + "\t" + ref + "\t" + alt + "\t" + "." + "\t" + "PASS" + "\t" + "SVTYPE=SNP" + "\t" + "GT:Contig" + "\t"  + "1/1" + ":" + contig_info +  "\n")
        count_id += 1

    for key, val in hetero.items():
        chr_num = str(val[0])
        start_ = val[1]
        end_ = val[2]
        ref = val[3]
        alt = val[4]
        GT = "0/1"
        contig_num_1 = val[5]
        contig_start_1 = val[6]
        contig_end_1 = val[7]
        contig_strand_dir_1 = val[8]
        contig_num_2 = val[9]
        contig_start_2 = val[10]
        contig_end_2 = val[11]
        contig_strand_dir_2 = val[12]
        new_contig_num_1 = contig_num_1.replace(":","_")
        new_contig_num_2 = contig_num_2.replace(":","_")
        contig_info = str(new_contig_num_1) + "_" + str(contig_start_1) + "_" + str(contig_end_1) + "_" + str(contig_strand_dir_1) + "_" +  str(new_contig_num_2) + "_" + str(contig_start_2) + "_" + str(contig_end_2) + "_" + str(contig_strand_dir_2) 

        fw.writelines(chr_num + "\t" + str(start_) + "\t" + "event" + str(count_id) + "\t" + ref + "\t" + alt + "\t" + "." + "\t" + "PASS" + "\t" + "SVTYPE=SNP" + "\t" + "GT:Contig" + "\t"  + "0/1"  + ":"  + contig_info + "\n")
        count_id += 1

    return (fw,count_id) 


if __name__ == "__main__":
    out_dir = args.out_dir
    if os.path.exists(out_dir):
        print("using existing output folder: " + out_dir)
    else:
        os.makedirs(out_dir)
    chr_start = args.chr_start
    chr_end = args.chr_end

    count_total_1 = 0
    count_total_2 = 0
    for chr_num in range(chr_start,chr_end + 1):
        SNV_dict_contig_1 = Extract_SNV_info(out_dir + "Aquila_Contig_chr" + str(chr_num) + "_hp1.var.txt",out_dir + "SNV_dict_contig_1_chr" + str(chr_num) + ".p",chr_num)
        SNV_dict_contig_2 = Extract_SNV_info(out_dir + "Aquila_Contig_chr" + str(chr_num) + "_hp2.var.txt",out_dir + "SNV_dict_contig_2_chr" + str(chr_num) + ".p",chr_num)
        print("chr" + str(chr_num) + ":")
        print(len(SNV_dict_contig_1))
        print(len(SNV_dict_contig_2))
        print("---------")
        count_total_1 += len(SNV_dict_contig_1)
        count_total_2 += len(SNV_dict_contig_2)

    print("total:")
    print(count_total_1)
    print(count_total_2)

    for chr_num in range(chr_start,chr_end + 1):
        ref_dict_contig_1 = Extract_ref_info(out_dir + "Aquila_Contig_chr" + str(chr_num) + "_hp1.var.txt",out_dir + "ref_dict_contig_1_chr" + str(chr_num) + ".p",chr_num)
        ref_dict_contig_2 = Extract_ref_info(out_dir + "Aquila_Contig_chr" + str(chr_num) + "_hp2.var.txt",out_dir + "ref_dict_contig_2_chr" + str(chr_num) + ".p",chr_num)

    count_hetero_total = 0
    count_hetero_1_total = 0
    count_hetero_2_total = 0
    count_hetero_final = 0
    count_same_total = 0
    count_same_wrong_total = 0
    for chr_num in range(chr_start,chr_end + 1):
        snv_homo,snv_hetero,count_same,count_hetero,count_same_wrong,count_hetero_1,count_hetero_2 = compare_two_haploid_SV(out_dir + "SNV_dict_contig_1_chr"+str(chr_num) + ".p", out_dir + "SNV_dict_contig_2_chr" + str(chr_num) + ".p", out_dir + "ref_dict_contig_1_chr" + str(chr_num) + ".p", out_dir + "ref_dict_contig_2_chr" + str(chr_num) + ".p", chr_num,out_dir)
        count_hetero_total += count_hetero
        count_hetero_1_total += count_hetero_1
        count_hetero_2_total += count_hetero_2
        count_hetero_final += (count_hetero_1 + count_hetero_2)
        count_same_total += count_same
        count_same_wrong_total += count_same_wrong
        print(count_hetero)
    print("total_hetero:")
    print(count_hetero_total)
    print(count_hetero_1_total)
    print(count_hetero_2_total)
    print(count_hetero_final)
    print("total_same:")
    print(count_same_total)
    print(count_same_wrong_total)
    vcf_output = out_dir + "Aquila_SNPs_chr" + str(chr_num) + ".vcf"
    count_id = 1
    fw = open(vcf_output,"w")
    #fw.writelines("#CHROM" + "\t" + "POS" + "\t" + "ID" + "\t" + "REF" + "\t" + "ALT" + "\t" + "QUAL" + "\t" + "FILTER" + "\t" + "INFO" + "\t" + "FORMAT" + "\t"  + "SAMPLE" + "\n")

    for chr_num in range(chr_start,chr_end + 1):
        fw,count_id = Write_vcf_for_SV(out_dir + "snv_hetero_chr" + str(chr_num) + ".p",out_dir + "snv_homo_chr" + str(chr_num) + ".p", fw,count_id)


















import pdb
#pdb.set_trace()
import glob
from collections import defaultdict
import os
from subprocess import Popen


def Merge_reads_for_PB_of_merged_libs(in_dir_list,out_dir,xin):
    if os.path.exists(out_dir):
        Popen("rm -rf " + out_dir,shell=True).wait()
        os.makedirs(out_dir)
    else:
        os.makedirs(out_dir)
    in_dir_list_raw = in_dir_list.split(",")
    in_dir_1 = in_dir_list_raw[0]
    fastq_files_all_1 = glob.glob(in_dir_1 +  "fastq_by_*.fastq")
    file_all_1 = []
    for one_file_raw in fastq_files_all_1:
        one_file = one_file_raw.split("/")[-1]
        file_all_1.append(one_file)
    print(len(file_all_1))
    merge_flag = defaultdict(int)
    count_merge = 0
    count_uniq_1 = 0 
    count_uniq_2 = 0
    for in_dir in in_dir_list_raw[1:]:
        fastq_files_all = glob.glob(in_dir +  "fastq_by_*.fastq")
        file_all = []
        for one_file_raw in fastq_files_all:
            one_file = one_file_raw.split("/")[-1]
            file_all.append(one_file)
            if one_file in file_all_1:
                rn_cmd = "mv " + in_dir_1 + one_file + " " + in_dir_1 + "temp.fastq"
                Popen(rn_cmd,shell=True).wait()
                cat_cmd = "cat " + in_dir_1 + "temp.fastq" + " " + in_dir + one_file + " > " + in_dir_1 + one_file
                Popen(cat_cmd,shell=True).wait()
            else:
                mv_cmd = "mv " + in_dir + one_file + " " + in_dir_1
                Popen(mv_cmd,shell=True).wait()

        ### update "in_dir_1" ###
        fastq_files_all_1 = glob.glob(in_dir_1 +  "fastq_by_*.fastq")
        file_all_1 = []
        for one_file_raw in fastq_files_all_1:
            one_file = one_file_raw.split("/")[-1]
            file_all_1.append(one_file)
        print(len(file_all_1))

    ### move final merged files ###
    mv_cmd = "mv " + in_dir_1 + "*" + " " + out_dir 
    Popen(mv_cmd,shell=True).wait()
    rm_cmd = "rm " + out_dir + "temp.fastq"
    Popen(rm_cmd,shell=True).wait()

    print("Merging Done~")





